# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: openhtf/output/proto/test_runs.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import descriptor_pb2 as google_dot_protobuf_dot_descriptor__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='openhtf/output/proto/test_runs.proto',
  package='openhtf',
  syntax='proto2',
  serialized_pb=_b('\n$openhtf/output/proto/test_runs.proto\x12\x07openhtf\x1a google/protobuf/descriptor.proto\",\n\x0b\x46\x61ilureCode\x12\x0c\n\x04\x63ode\x18\x01 \x01(\t\x12\x0f\n\x07\x64\x65tails\x18\x02 \x01(\t\">\n\x08TimeInfo\x12\x19\n\x11start_time_millis\x18\x01 \x01(\x03\x12\x17\n\x0f\x65nd_time_millis\x18\x02 \x01(\x03\"Q\n\x06Timing\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x02 \x01(\t\x12$\n\ttime_info\x18\x03 \x01(\x0b\x32\x11.openhtf.TimeInfo\"c\n\x05Phase\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x02 \x01(\t\x12\x14\n\x0c\x63\x61pabilities\x18\x03 \x03(\t\x12!\n\x06timing\x18\x04 \x01(\x0b\x32\x11.openhtf.TimeInfo\"\xf2\x02\n\rTestParameter\x12\x0c\n\x04name\x18\x01 \x02(\t\x12\x1f\n\x06status\x18\x02 \x01(\x0e\x32\x0f.openhtf.Status\x12\x13\n\x0b\x64\x65scription\x18\x06 \x01(\t\x12\x11\n\timportant\x18\x12 \x01(\x08\x12\x15\n\rparameter_tag\x18\x10 \x01(\t\x12\x15\n\rnumeric_value\x18\x0b \x01(\x01\x12\x17\n\x0fnumeric_minimum\x18\x0c \x01(\x01\x12\x17\n\x0fnumeric_maximum\x18\r \x01(\x01\x12\x12\n\ntext_value\x18\x0e \x01(\t\x12\x15\n\rexpected_text\x18\x0f \x01(\t\x12\x13\n\x0bis_optional\x18\x11 \x01(\x08\x12\x17\n\x0fset_time_millis\x18\x13 \x01(\x03\x12*\n\tunit_code\x18\x14 \x01(\x0e\x32\x17.openhtf.Units.UnitCode\x12\x1d\n\x15\x61ssociated_attachment\x18\x15 \x01(\t*\x06\x08\x88\'\x10\xd0(\"\x81\x02\n\x11TestRunLogMessage\x12\x18\n\x10timestamp_millis\x18\x01 \x01(\x03\x12\x13\n\x0blog_message\x18\x02 \x01(\t\x12\x13\n\x0blogger_name\x18\x03 \x01(\t\x12\x0f\n\x07levelno\x18\x04 \x01(\x05\x12/\n\x05level\x18\x07 \x01(\x0e\x32 .openhtf.TestRunLogMessage.Level\x12\x12\n\nlog_source\x18\x05 \x01(\t\x12\x0e\n\x06lineno\x18\x06 \x01(\x05\"B\n\x05Level\x12\t\n\x05\x44\x45\x42UG\x10\n\x12\x08\n\x04INFO\x10\x14\x12\x0b\n\x07WARNING\x10\x1e\x12\t\n\x05\x45RROR\x10(\x12\x0c\n\x08\x43RITICAL\x10\x32\"\xc0\x04\n\x07TestRun\x12\x12\n\ndut_serial\x18\x01 \x02(\t\x12\x13\n\x0btester_name\x18\x02 \x02(\t\x12$\n\ttest_info\x18\x03 \x02(\x0b\x32\x11.openhtf.TestInfo\x12-\n\x0btest_status\x18\x04 \x02(\x0e\x32\x0f.openhtf.Status:\x07\x43REATED\x12\x19\n\x11start_time_millis\x18\x08 \x01(\x03\x12\x17\n\x0f\x65nd_time_millis\x18\t \x01(\x03\x12\x10\n\x08run_name\x18\n \x01(\t\x12/\n\x0ftest_parameters\x18\x05 \x03(\x0b\x32\x16.openhtf.TestParameter\x12\x36\n\x0finfo_parameters\x18\x06 \x03(\x0b\x32\x1d.openhtf.InformationParameter\x12-\n\ttest_logs\x18\x0b \x03(\x0b\x32\x1a.openhtf.TestRunLogMessage\x12+\n\rfailure_codes\x18\x13 \x03(\x0b\x32\x14.openhtf.FailureCode\x12\x15\n\roperator_name\x18\x16 \x01(\t\x12\x12\n\nlot_number\x18\x17 \x01(\t\x12\x0f\n\x07part_id\x18\x18 \x01(\t\x12\x15\n\rsynthetic_dut\x18\x19 \x01(\x08\x12 \n\x07timings\x18\x1b \x03(\x0b\x32\x0f.openhtf.Timing\x12\x1e\n\x06phases\x18\x1c \x03(\x0b\x32\x0e.openhtf.Phase\x12\x17\n\x0f\x66ramework_build\x18\x11 \x01(\t\"E\n\x08TestInfo\x12\x0c\n\x04name\x18\x01 \x02(\t\x12\x13\n\x0b\x64\x65scription\x18\x02 \x01(\t\x12\x16\n\x0eversion_string\x18\x05 \x01(\t\"\xb1\x01\n\x14InformationParameter\x12\x0c\n\x04name\x18\x01 \x02(\t\x12\x14\n\x0cvalue_binary\x18\x07 \x01(\x0c\x12\x13\n\x0b\x64\x65scription\x18\x03 \x01(\t\x12\x17\n\x0fset_time_millis\x18\x08 \x01(\x03\x12\x15\n\rparameter_tag\x18\x05 \x01(\t\x12\x30\n\x04type\x18\x04 \x01(\x0e\x32\x17.openhtf.InformationTag:\tTEXT_UTF8\"\xa4\t\n\x05Units\"\x9a\t\n\x08UnitCode\x12\x08\n\x04NONE\x10\x01\x12\x1c\n\x07PERCENT\x10\x02\x1a\x0f\xd2\xb0\xd6\xc0\x01\x02P1\x9a\xd0\x86\xfd\x01\x01%\x12\x1b\n\x0cNO_DIMENSION\x10\x03\x1a\t\xd2\xb0\xd6\xc0\x01\x03NDL\x12\x1b\n\x05PIXEL\x10\x04\x1a\x10\xd2\xb0\xd6\xc0\x01\x02PX\x9a\xd0\x86\xfd\x01\x02px\x12\x1a\n\x0bPIXEL_LEVEL\x10\x05\x1a\t\xd2\xb0\xd6\xc0\x01\x03PXL\x12,\n\x14ROTATIONS_PER_MINUTE\x10\x07\x1a\x12\xd2\xb0\xd6\xc0\x01\x03RPM\x9a\xd0\x86\xfd\x01\x03rpm\x12\x1c\n\x06SECOND\x10\n\x1a\x10\xd2\xb0\xd6\xc0\x01\x03SEC\x9a\xd0\x86\xfd\x01\x01s\x12\x1b\n\x03MHZ\x10\x0b\x1a\x12\xd2\xb0\xd6\xc0\x01\x03MHZ\x9a\xd0\x86\xfd\x01\x03MHz\x12\x1c\n\x05HERTZ\x10\x0c\x1a\x11\xd2\xb0\xd6\xc0\x01\x03HTZ\x9a\xd0\x86\xfd\x01\x02Hz\x12#\n\x0bMICROSECOND\x10\r\x1a\x12\xd2\xb0\xd6\xc0\x01\x03\x42\x39\x38\x9a\xd0\x86\xfd\x01\x03\xc2\xb5s\x12!\n\nMILLIMETER\x10\x15\x1a\x11\xd2\xb0\xd6\xc0\x01\x03MMT\x9a\xd0\x86\xfd\x01\x02mm\x12 \n\nCENTIMETER\x10\x16\x1a\x10\xd2\xb0\xd6\xc0\x01\x02LC\x9a\xd0\x86\xfd\x01\x02\x63m\x12\x1b\n\x05METER\x10\x17\x1a\x10\xd2\xb0\xd6\xc0\x01\x03MTR\x9a\xd0\x86\xfd\x01\x01m\x12$\n\tPER_METER\x10\x18\x1a\x15\xd2\xb0\xd6\xc0\x01\x03M0R\x9a\xd0\x86\xfd\x01\x06m\xe2\x81\xbb\xc2\xb9\x12!\n\nMILLILITER\x10\x19\x1a\x11\xd2\xb0\xd6\xc0\x01\x03MLT\x9a\xd0\x86\xfd\x01\x02mL\x12#\n\nCUBIC_FOOT\x10\x1a\x1a\x13\xd2\xb0\xd6\xc0\x01\x03MTQ\x9a\xd0\x86\xfd\x01\x04\x46t\xc2\xb3\x12\x1d\n\x07\x44\x45\x43IBEL\x10\x1e\x1a\x10\xd2\xb0\xd6\xc0\x01\x02\x32N\x9a\xd0\x86\xfd\x01\x02\x64\x42\x12\"\n\nDECIBEL_MW\x10\x1f\x1a\x12\xd2\xb0\xd6\xc0\x01\x02\x32N\x9a\xd0\x86\xfd\x01\x04\x64\x42mW\x12 \n\x08MICROAMP\x10 \x1a\x12\xd2\xb0\xd6\xc0\x01\x03\x42\x38\x34\x9a\xd0\x86\xfd\x01\x03\xc2\xb5\x41\x12\x1e\n\x08MILLIAMP\x10!\x1a\x10\xd2\xb0\xd6\xc0\x01\x02\x34K\x9a\xd0\x86\xfd\x01\x02mA\x12!\n\tMICROVOLT\x10\"\x1a\x12\xd2\xb0\xd6\xc0\x01\x03\x44\x38\x32\x9a\xd0\x86\xfd\x01\x03\xc2\xb5V\x12\x1a\n\x04VOLT\x10#\x1a\x10\xd2\xb0\xd6\xc0\x01\x03VLT\x9a\xd0\x86\xfd\x01\x01V\x12\x1f\n\tPICOFARAD\x10$\x1a\x10\xd2\xb0\xd6\xc0\x01\x02\x34T\x9a\xd0\x86\xfd\x01\x02pF\x12\x16\n\x07\x43OULOMB\x10%\x1a\t\xd2\xb0\xd6\xc0\x01\x03\x43OU\x12\x1f\n\tMILLIVOLT\x10&\x1a\x10\xd2\xb0\xd6\xc0\x01\x02\x32Z\x9a\xd0\x86\xfd\x01\x02mV\x12\x1a\n\x04WATT\x10\'\x1a\x10\xd2\xb0\xd6\xc0\x01\x03WTT\x9a\xd0\x86\xfd\x01\x01W\x12\x1c\n\x06\x41MPERE\x10\x1d\x1a\x10\xd2\xb0\xd6\xc0\x01\x03\x41MP\x9a\xd0\x86\xfd\x01\x01\x41\x12&\n\x0e\x44\x45GREE_CELSIUS\x10(\x1a\x12\xd2\xb0\xd6\xc0\x01\x03\x43\x45L\x9a\xd0\x86\xfd\x01\x03\xc2\xb0\x43\x12\x1c\n\x06KELVIN\x10)\x1a\x10\xd2\xb0\xd6\xc0\x01\x03KEL\x9a\xd0\x86\xfd\x01\x01K\x12\x12\n\x04\x42YTE\x10\x32\x1a\x08\xd2\xb0\xd6\xc0\x01\x02\x41\x44\x12/\n\x15MEGA_BYTES_PER_SECOND\x10\x33\x1a\x14\xd2\xb0\xd6\xc0\x01\x04MBPS\x9a\xd0\x86\xfd\x01\x04MB/s\x12\x1c\n\x06\x44\x45GREE\x10<\x1a\x10\xd2\xb0\xd6\xc0\x01\x02\x44\x44\x9a\xd0\x86\xfd\x01\x02\xc2\xb0\x12\x1e\n\x06RADIAN\x10=\x1a\x12\xd2\xb0\xd6\xc0\x01\x03\x43\x38\x31\x9a\xd0\x86\xfd\x01\x03rad\x12\x1c\n\x06NEWTON\x10\x46\x1a\x10\xd2\xb0\xd6\xc0\x01\x03NEW\x9a\xd0\x86\xfd\x01\x01N\x12\x32\n\x18\x43UBIC_CENTIMETER_PER_SEC\x10P\x1a\x14\xd2\xb0\xd6\xc0\x01\x02\x32J\x9a\xd0\x86\xfd\x01\x06\x63m\xc2\xb3/s\x12!\n\x08MILLIBAR\x10Q\x1a\x13\xd2\xb0\xd6\xc0\x01\x03MBR\x9a\xd0\x86\xfd\x01\x04mbar*\x85\x01\n\x0eInformationTag\x12\x07\n\x03JPG\x10\x02\x12\x07\n\x03PNG\x10\x03\x12\x07\n\x03WAV\x10\x04\x12\x08\n\x04TIFF\x10\x08\x12\x07\n\x03MP4\x10\t\x12\x07\n\x03\x41VI\x10\x0b\x12\n\n\x06\x42INARY\x10\x05\x12\x0e\n\nTIMESERIES\x10\x06\x12\r\n\tTEXT_UTF8\x10\x07\x12\x11\n\rMULTIDIM_JSON\x10\n*\xa0\x01\n\x06Status\x12\x08\n\x04PASS\x10\x01\x12\x08\n\x04\x46\x41IL\x10\x02\x12\t\n\x05\x45RROR\x10\x03\x12\x0b\n\x07RUNNING\x10\x04\x12\x0b\n\x07\x43REATED\x10\x05\x12\x0b\n\x07TIMEOUT\x10\x06\x12\x0b\n\x07\x41\x42ORTED\x10\x07\x12\x0b\n\x07WAITING\x10\x08\x12\x0b\n\x07\x43ONSUME\x10\n\x12\x07\n\x03RMA\x10\x0b\x12\n\n\x06REWORK\x10\x0c\x12\t\n\x05SCRAP\x10\r\x12\t\n\x05\x44\x45\x42UG\x10\x0e:6\n\x08uom_code\x12!.google.protobuf.EnumValueOptions\x18\x8a\xe6\x8a\x18 \x01(\t:8\n\nuom_suffix\x12!.google.protobuf.EnumValueOptions\x18\x83\xea\xd0\x1f \x01(\t')
  ,
  dependencies=[google_dot_protobuf_dot_descriptor__pb2.DESCRIPTOR,])
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

_INFORMATIONTAG = _descriptor.EnumDescriptor(
  name='InformationTag',
  full_name='openhtf.InformationTag',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='JPG', index=0, number=2,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='PNG', index=1, number=3,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='WAV', index=2, number=4,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='TIFF', index=3, number=8,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='MP4', index=4, number=9,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='AVI', index=5, number=11,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='BINARY', index=6, number=5,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='TIMESERIES', index=7, number=6,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='TEXT_UTF8', index=8, number=7,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='MULTIDIM_JSON', index=9, number=10,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=3032,
  serialized_end=3165,
)
_sym_db.RegisterEnumDescriptor(_INFORMATIONTAG)

InformationTag = enum_type_wrapper.EnumTypeWrapper(_INFORMATIONTAG)
_STATUS = _descriptor.EnumDescriptor(
  name='Status',
  full_name='openhtf.Status',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='PASS', index=0, number=1,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='FAIL', index=1, number=2,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='ERROR', index=2, number=3,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='RUNNING', index=3, number=4,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='CREATED', index=4, number=5,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='TIMEOUT', index=5, number=6,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='ABORTED', index=6, number=7,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='WAITING', index=7, number=8,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='CONSUME', index=8, number=10,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='RMA', index=9, number=11,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='REWORK', index=10, number=12,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='SCRAP', index=11, number=13,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='DEBUG', index=12, number=14,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=3168,
  serialized_end=3328,
)
_sym_db.RegisterEnumDescriptor(_STATUS)

Status = enum_type_wrapper.EnumTypeWrapper(_STATUS)
JPG = 2
PNG = 3
WAV = 4
TIFF = 8
MP4 = 9
AVI = 11
BINARY = 5
TIMESERIES = 6
TEXT_UTF8 = 7
MULTIDIM_JSON = 10
PASS = 1
FAIL = 2
ERROR = 3
RUNNING = 4
CREATED = 5
TIMEOUT = 6
ABORTED = 7
WAITING = 8
CONSUME = 10
RMA = 11
REWORK = 12
SCRAP = 13
DEBUG = 14

UOM_CODE_FIELD_NUMBER = 50508554
uom_code = _descriptor.FieldDescriptor(
  name='uom_code', full_name='openhtf.uom_code', index=0,
  number=50508554, type=9, cpp_type=9, label=1,
  has_default_value=False, default_value=_b("").decode('utf-8'),
  message_type=None, enum_type=None, containing_type=None,
  is_extension=True, extension_scope=None,
  options=None)
UOM_SUFFIX_FIELD_NUMBER = 66336003
uom_suffix = _descriptor.FieldDescriptor(
  name='uom_suffix', full_name='openhtf.uom_suffix', index=1,
  number=66336003, type=9, cpp_type=9, label=1,
  has_default_value=False, default_value=_b("").decode('utf-8'),
  message_type=None, enum_type=None, containing_type=None,
  is_extension=True, extension_scope=None,
  options=None)

_TESTRUNLOGMESSAGE_LEVEL = _descriptor.EnumDescriptor(
  name='Level',
  full_name='openhtf.TestRunLogMessage.Level',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='DEBUG', index=0, number=10,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='INFO', index=1, number=20,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='WARNING', index=2, number=30,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='ERROR', index=3, number=40,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='CRITICAL', index=4, number=50,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=942,
  serialized_end=1008,
)
_sym_db.RegisterEnumDescriptor(_TESTRUNLOGMESSAGE_LEVEL)

_UNITS_UNITCODE = _descriptor.EnumDescriptor(
  name='UnitCode',
  full_name='openhtf.Units.UnitCode',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='NONE', index=0, number=1,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='PERCENT', index=1, number=2,
      options=_descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\322\260\326\300\001\002P1\232\320\206\375\001\001%')),
      type=None),
    _descriptor.EnumValueDescriptor(
      name='NO_DIMENSION', index=2, number=3,
      options=_descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\322\260\326\300\001\003NDL')),
      type=None),
    _descriptor.EnumValueDescriptor(
      name='PIXEL', index=3, number=4,
      options=_descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\322\260\326\300\001\002PX\232\320\206\375\001\002px')),
      type=None),
    _descriptor.EnumValueDescriptor(
      name='PIXEL_LEVEL', index=4, number=5,
      options=_descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\322\260\326\300\001\003PXL')),
      type=None),
    _descriptor.EnumValueDescriptor(
      name='ROTATIONS_PER_MINUTE', index=5, number=7,
      options=_descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\322\260\326\300\001\003RPM\232\320\206\375\001\003rpm')),
      type=None),
    _descriptor.EnumValueDescriptor(
      name='SECOND', index=6, number=10,
      options=_descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\322\260\326\300\001\003SEC\232\320\206\375\001\001s')),
      type=None),
    _descriptor.EnumValueDescriptor(
      name='MHZ', index=7, number=11,
      options=_descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\322\260\326\300\001\003MHZ\232\320\206\375\001\003MHz')),
      type=None),
    _descriptor.EnumValueDescriptor(
      name='HERTZ', index=8, number=12,
      options=_descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\322\260\326\300\001\003HTZ\232\320\206\375\001\002Hz')),
      type=None),
    _descriptor.EnumValueDescriptor(
      name='MICROSECOND', index=9, number=13,
      options=_descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\322\260\326\300\001\003B98\232\320\206\375\001\003\302\265s')),
      type=None),
    _descriptor.EnumValueDescriptor(
      name='MILLIMETER', index=10, number=21,
      options=_descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\322\260\326\300\001\003MMT\232\320\206\375\001\002mm')),
      type=None),
    _descriptor.EnumValueDescriptor(
      name='CENTIMETER', index=11, number=22,
      options=_descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\322\260\326\300\001\002LC\232\320\206\375\001\002cm')),
      type=None),
    _descriptor.EnumValueDescriptor(
      name='METER', index=12, number=23,
      options=_descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\322\260\326\300\001\003MTR\232\320\206\375\001\001m')),
      type=None),
    _descriptor.EnumValueDescriptor(
      name='PER_METER', index=13, number=24,
      options=_descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\322\260\326\300\001\003M0R\232\320\206\375\001\006m\342\201\273\302\271')),
      type=None),
    _descriptor.EnumValueDescriptor(
      name='MILLILITER', index=14, number=25,
      options=_descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\322\260\326\300\001\003MLT\232\320\206\375\001\002mL')),
      type=None),
    _descriptor.EnumValueDescriptor(
      name='CUBIC_FOOT', index=15, number=26,
      options=_descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\322\260\326\300\001\003MTQ\232\320\206\375\001\004Ft\302\263')),
      type=None),
    _descriptor.EnumValueDescriptor(
      name='DECIBEL', index=16, number=30,
      options=_descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\322\260\326\300\001\0022N\232\320\206\375\001\002dB')),
      type=None),
    _descriptor.EnumValueDescriptor(
      name='DECIBEL_MW', index=17, number=31,
      options=_descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\322\260\326\300\001\0022N\232\320\206\375\001\004dBmW')),
      type=None),
    _descriptor.EnumValueDescriptor(
      name='MICROAMP', index=18, number=32,
      options=_descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\322\260\326\300\001\003B84\232\320\206\375\001\003\302\265A')),
      type=None),
    _descriptor.EnumValueDescriptor(
      name='MILLIAMP', index=19, number=33,
      options=_descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\322\260\326\300\001\0024K\232\320\206\375\001\002mA')),
      type=None),
    _descriptor.EnumValueDescriptor(
      name='MICROVOLT', index=20, number=34,
      options=_descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\322\260\326\300\001\003D82\232\320\206\375\001\003\302\265V')),
      type=None),
    _descriptor.EnumValueDescriptor(
      name='VOLT', index=21, number=35,
      options=_descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\322\260\326\300\001\003VLT\232\320\206\375\001\001V')),
      type=None),
    _descriptor.EnumValueDescriptor(
      name='PICOFARAD', index=22, number=36,
      options=_descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\322\260\326\300\001\0024T\232\320\206\375\001\002pF')),
      type=None),
    _descriptor.EnumValueDescriptor(
      name='COULOMB', index=23, number=37,
      options=_descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\322\260\326\300\001\003COU')),
      type=None),
    _descriptor.EnumValueDescriptor(
      name='MILLIVOLT', index=24, number=38,
      options=_descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\322\260\326\300\001\0022Z\232\320\206\375\001\002mV')),
      type=None),
    _descriptor.EnumValueDescriptor(
      name='WATT', index=25, number=39,
      options=_descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\322\260\326\300\001\003WTT\232\320\206\375\001\001W')),
      type=None),
    _descriptor.EnumValueDescriptor(
      name='AMPERE', index=26, number=29,
      options=_descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\322\260\326\300\001\003AMP\232\320\206\375\001\001A')),
      type=None),
    _descriptor.EnumValueDescriptor(
      name='DEGREE_CELSIUS', index=27, number=40,
      options=_descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\322\260\326\300\001\003CEL\232\320\206\375\001\003\302\260C')),
      type=None),
    _descriptor.EnumValueDescriptor(
      name='KELVIN', index=28, number=41,
      options=_descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\322\260\326\300\001\003KEL\232\320\206\375\001\001K')),
      type=None),
    _descriptor.EnumValueDescriptor(
      name='BYTE', index=29, number=50,
      options=_descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\322\260\326\300\001\002AD')),
      type=None),
    _descriptor.EnumValueDescriptor(
      name='MEGA_BYTES_PER_SECOND', index=30, number=51,
      options=_descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\322\260\326\300\001\004MBPS\232\320\206\375\001\004MB/s')),
      type=None),
    _descriptor.EnumValueDescriptor(
      name='DEGREE', index=31, number=60,
      options=_descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\322\260\326\300\001\002DD\232\320\206\375\001\002\302\260')),
      type=None),
    _descriptor.EnumValueDescriptor(
      name='RADIAN', index=32, number=61,
      options=_descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\322\260\326\300\001\003C81\232\320\206\375\001\003rad')),
      type=None),
    _descriptor.EnumValueDescriptor(
      name='NEWTON', index=33, number=70,
      options=_descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\322\260\326\300\001\003NEW\232\320\206\375\001\001N')),
      type=None),
    _descriptor.EnumValueDescriptor(
      name='CUBIC_CENTIMETER_PER_SEC', index=34, number=80,
      options=_descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\322\260\326\300\001\0022J\232\320\206\375\001\006cm\302\263/s')),
      type=None),
    _descriptor.EnumValueDescriptor(
      name='MILLIBAR', index=35, number=81,
      options=_descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\322\260\326\300\001\003MBR\232\320\206\375\001\004mbar')),
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=1851,
  serialized_end=3029,
)
_sym_db.RegisterEnumDescriptor(_UNITS_UNITCODE)


_FAILURECODE = _descriptor.Descriptor(
  name='FailureCode',
  full_name='openhtf.FailureCode',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='code', full_name='openhtf.FailureCode.code', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='details', full_name='openhtf.FailureCode.details', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=83,
  serialized_end=127,
)


_TIMEINFO = _descriptor.Descriptor(
  name='TimeInfo',
  full_name='openhtf.TimeInfo',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='start_time_millis', full_name='openhtf.TimeInfo.start_time_millis', index=0,
      number=1, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='end_time_millis', full_name='openhtf.TimeInfo.end_time_millis', index=1,
      number=2, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=129,
  serialized_end=191,
)


_TIMING = _descriptor.Descriptor(
  name='Timing',
  full_name='openhtf.Timing',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='openhtf.Timing.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='description', full_name='openhtf.Timing.description', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='time_info', full_name='openhtf.Timing.time_info', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=193,
  serialized_end=274,
)


_PHASE = _descriptor.Descriptor(
  name='Phase',
  full_name='openhtf.Phase',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='openhtf.Phase.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='description', full_name='openhtf.Phase.description', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='capabilities', full_name='openhtf.Phase.capabilities', index=2,
      number=3, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='timing', full_name='openhtf.Phase.timing', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=276,
  serialized_end=375,
)


_TESTPARAMETER = _descriptor.Descriptor(
  name='TestParameter',
  full_name='openhtf.TestParameter',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='openhtf.TestParameter.name', index=0,
      number=1, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='status', full_name='openhtf.TestParameter.status', index=1,
      number=2, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=1,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='description', full_name='openhtf.TestParameter.description', index=2,
      number=6, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='important', full_name='openhtf.TestParameter.important', index=3,
      number=18, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='parameter_tag', full_name='openhtf.TestParameter.parameter_tag', index=4,
      number=16, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='numeric_value', full_name='openhtf.TestParameter.numeric_value', index=5,
      number=11, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='numeric_minimum', full_name='openhtf.TestParameter.numeric_minimum', index=6,
      number=12, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='numeric_maximum', full_name='openhtf.TestParameter.numeric_maximum', index=7,
      number=13, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='text_value', full_name='openhtf.TestParameter.text_value', index=8,
      number=14, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='expected_text', full_name='openhtf.TestParameter.expected_text', index=9,
      number=15, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='is_optional', full_name='openhtf.TestParameter.is_optional', index=10,
      number=17, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='set_time_millis', full_name='openhtf.TestParameter.set_time_millis', index=11,
      number=19, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='unit_code', full_name='openhtf.TestParameter.unit_code', index=12,
      number=20, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=1,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='associated_attachment', full_name='openhtf.TestParameter.associated_attachment', index=13,
      number=21, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=True,
  syntax='proto2',
  extension_ranges=[(5000, 5200), ],
  oneofs=[
  ],
  serialized_start=378,
  serialized_end=748,
)


_TESTRUNLOGMESSAGE = _descriptor.Descriptor(
  name='TestRunLogMessage',
  full_name='openhtf.TestRunLogMessage',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='timestamp_millis', full_name='openhtf.TestRunLogMessage.timestamp_millis', index=0,
      number=1, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='log_message', full_name='openhtf.TestRunLogMessage.log_message', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='logger_name', full_name='openhtf.TestRunLogMessage.logger_name', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='levelno', full_name='openhtf.TestRunLogMessage.levelno', index=3,
      number=4, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='level', full_name='openhtf.TestRunLogMessage.level', index=4,
      number=7, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=10,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='log_source', full_name='openhtf.TestRunLogMessage.log_source', index=5,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='lineno', full_name='openhtf.TestRunLogMessage.lineno', index=6,
      number=6, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _TESTRUNLOGMESSAGE_LEVEL,
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=751,
  serialized_end=1008,
)


_TESTRUN = _descriptor.Descriptor(
  name='TestRun',
  full_name='openhtf.TestRun',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='dut_serial', full_name='openhtf.TestRun.dut_serial', index=0,
      number=1, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='tester_name', full_name='openhtf.TestRun.tester_name', index=1,
      number=2, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='test_info', full_name='openhtf.TestRun.test_info', index=2,
      number=3, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='test_status', full_name='openhtf.TestRun.test_status', index=3,
      number=4, type=14, cpp_type=8, label=2,
      has_default_value=True, default_value=5,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='start_time_millis', full_name='openhtf.TestRun.start_time_millis', index=4,
      number=8, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='end_time_millis', full_name='openhtf.TestRun.end_time_millis', index=5,
      number=9, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='run_name', full_name='openhtf.TestRun.run_name', index=6,
      number=10, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='test_parameters', full_name='openhtf.TestRun.test_parameters', index=7,
      number=5, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='info_parameters', full_name='openhtf.TestRun.info_parameters', index=8,
      number=6, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='test_logs', full_name='openhtf.TestRun.test_logs', index=9,
      number=11, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='failure_codes', full_name='openhtf.TestRun.failure_codes', index=10,
      number=19, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='operator_name', full_name='openhtf.TestRun.operator_name', index=11,
      number=22, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='lot_number', full_name='openhtf.TestRun.lot_number', index=12,
      number=23, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='part_id', full_name='openhtf.TestRun.part_id', index=13,
      number=24, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='synthetic_dut', full_name='openhtf.TestRun.synthetic_dut', index=14,
      number=25, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='timings', full_name='openhtf.TestRun.timings', index=15,
      number=27, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='phases', full_name='openhtf.TestRun.phases', index=16,
      number=28, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='framework_build', full_name='openhtf.TestRun.framework_build', index=17,
      number=17, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1011,
  serialized_end=1587,
)


_TESTINFO = _descriptor.Descriptor(
  name='TestInfo',
  full_name='openhtf.TestInfo',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='openhtf.TestInfo.name', index=0,
      number=1, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='description', full_name='openhtf.TestInfo.description', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='version_string', full_name='openhtf.TestInfo.version_string', index=2,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1589,
  serialized_end=1658,
)


_INFORMATIONPARAMETER = _descriptor.Descriptor(
  name='InformationParameter',
  full_name='openhtf.InformationParameter',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='openhtf.InformationParameter.name', index=0,
      number=1, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='value_binary', full_name='openhtf.InformationParameter.value_binary', index=1,
      number=7, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='description', full_name='openhtf.InformationParameter.description', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='set_time_millis', full_name='openhtf.InformationParameter.set_time_millis', index=3,
      number=8, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='parameter_tag', full_name='openhtf.InformationParameter.parameter_tag', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='type', full_name='openhtf.InformationParameter.type', index=5,
      number=4, type=14, cpp_type=8, label=1,
      has_default_value=True, default_value=7,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1661,
  serialized_end=1838,
)


_UNITS = _descriptor.Descriptor(
  name='Units',
  full_name='openhtf.Units',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _UNITS_UNITCODE,
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1841,
  serialized_end=3029,
)

_TIMING.fields_by_name['time_info'].message_type = _TIMEINFO
_PHASE.fields_by_name['timing'].message_type = _TIMEINFO
_TESTPARAMETER.fields_by_name['status'].enum_type = _STATUS
_TESTPARAMETER.fields_by_name['unit_code'].enum_type = _UNITS_UNITCODE
_TESTRUNLOGMESSAGE.fields_by_name['level'].enum_type = _TESTRUNLOGMESSAGE_LEVEL
_TESTRUNLOGMESSAGE_LEVEL.containing_type = _TESTRUNLOGMESSAGE
_TESTRUN.fields_by_name['test_info'].message_type = _TESTINFO
_TESTRUN.fields_by_name['test_status'].enum_type = _STATUS
_TESTRUN.fields_by_name['test_parameters'].message_type = _TESTPARAMETER
_TESTRUN.fields_by_name['info_parameters'].message_type = _INFORMATIONPARAMETER
_TESTRUN.fields_by_name['test_logs'].message_type = _TESTRUNLOGMESSAGE
_TESTRUN.fields_by_name['failure_codes'].message_type = _FAILURECODE
_TESTRUN.fields_by_name['timings'].message_type = _TIMING
_TESTRUN.fields_by_name['phases'].message_type = _PHASE
_INFORMATIONPARAMETER.fields_by_name['type'].enum_type = _INFORMATIONTAG
_UNITS_UNITCODE.containing_type = _UNITS
DESCRIPTOR.message_types_by_name['FailureCode'] = _FAILURECODE
DESCRIPTOR.message_types_by_name['TimeInfo'] = _TIMEINFO
DESCRIPTOR.message_types_by_name['Timing'] = _TIMING
DESCRIPTOR.message_types_by_name['Phase'] = _PHASE
DESCRIPTOR.message_types_by_name['TestParameter'] = _TESTPARAMETER
DESCRIPTOR.message_types_by_name['TestRunLogMessage'] = _TESTRUNLOGMESSAGE
DESCRIPTOR.message_types_by_name['TestRun'] = _TESTRUN
DESCRIPTOR.message_types_by_name['TestInfo'] = _TESTINFO
DESCRIPTOR.message_types_by_name['InformationParameter'] = _INFORMATIONPARAMETER
DESCRIPTOR.message_types_by_name['Units'] = _UNITS
DESCRIPTOR.enum_types_by_name['InformationTag'] = _INFORMATIONTAG
DESCRIPTOR.enum_types_by_name['Status'] = _STATUS
DESCRIPTOR.extensions_by_name['uom_code'] = uom_code
DESCRIPTOR.extensions_by_name['uom_suffix'] = uom_suffix

FailureCode = _reflection.GeneratedProtocolMessageType('FailureCode', (_message.Message,), dict(
  DESCRIPTOR = _FAILURECODE,
  __module__ = 'openhtf.output.proto.test_runs_pb2'
  # @@protoc_insertion_point(class_scope:openhtf.FailureCode)
  ))
_sym_db.RegisterMessage(FailureCode)

TimeInfo = _reflection.GeneratedProtocolMessageType('TimeInfo', (_message.Message,), dict(
  DESCRIPTOR = _TIMEINFO,
  __module__ = 'openhtf.output.proto.test_runs_pb2'
  # @@protoc_insertion_point(class_scope:openhtf.TimeInfo)
  ))
_sym_db.RegisterMessage(TimeInfo)

Timing = _reflection.GeneratedProtocolMessageType('Timing', (_message.Message,), dict(
  DESCRIPTOR = _TIMING,
  __module__ = 'openhtf.output.proto.test_runs_pb2'
  # @@protoc_insertion_point(class_scope:openhtf.Timing)
  ))
_sym_db.RegisterMessage(Timing)

Phase = _reflection.GeneratedProtocolMessageType('Phase', (_message.Message,), dict(
  DESCRIPTOR = _PHASE,
  __module__ = 'openhtf.output.proto.test_runs_pb2'
  # @@protoc_insertion_point(class_scope:openhtf.Phase)
  ))
_sym_db.RegisterMessage(Phase)

TestParameter = _reflection.GeneratedProtocolMessageType('TestParameter', (_message.Message,), dict(
  DESCRIPTOR = _TESTPARAMETER,
  __module__ = 'openhtf.output.proto.test_runs_pb2'
  # @@protoc_insertion_point(class_scope:openhtf.TestParameter)
  ))
_sym_db.RegisterMessage(TestParameter)

TestRunLogMessage = _reflection.GeneratedProtocolMessageType('TestRunLogMessage', (_message.Message,), dict(
  DESCRIPTOR = _TESTRUNLOGMESSAGE,
  __module__ = 'openhtf.output.proto.test_runs_pb2'
  # @@protoc_insertion_point(class_scope:openhtf.TestRunLogMessage)
  ))
_sym_db.RegisterMessage(TestRunLogMessage)

TestRun = _reflection.GeneratedProtocolMessageType('TestRun', (_message.Message,), dict(
  DESCRIPTOR = _TESTRUN,
  __module__ = 'openhtf.output.proto.test_runs_pb2'
  # @@protoc_insertion_point(class_scope:openhtf.TestRun)
  ))
_sym_db.RegisterMessage(TestRun)

TestInfo = _reflection.GeneratedProtocolMessageType('TestInfo', (_message.Message,), dict(
  DESCRIPTOR = _TESTINFO,
  __module__ = 'openhtf.output.proto.test_runs_pb2'
  # @@protoc_insertion_point(class_scope:openhtf.TestInfo)
  ))
_sym_db.RegisterMessage(TestInfo)

InformationParameter = _reflection.GeneratedProtocolMessageType('InformationParameter', (_message.Message,), dict(
  DESCRIPTOR = _INFORMATIONPARAMETER,
  __module__ = 'openhtf.output.proto.test_runs_pb2'
  # @@protoc_insertion_point(class_scope:openhtf.InformationParameter)
  ))
_sym_db.RegisterMessage(InformationParameter)

Units = _reflection.GeneratedProtocolMessageType('Units', (_message.Message,), dict(
  DESCRIPTOR = _UNITS,
  __module__ = 'openhtf.output.proto.test_runs_pb2'
  # @@protoc_insertion_point(class_scope:openhtf.Units)
  ))
_sym_db.RegisterMessage(Units)

google_dot_protobuf_dot_descriptor__pb2.EnumValueOptions.RegisterExtension(uom_code)
google_dot_protobuf_dot_descriptor__pb2.EnumValueOptions.RegisterExtension(uom_suffix)

_UNITS_UNITCODE.values_by_name["PERCENT"].has_options = True
_UNITS_UNITCODE.values_by_name["PERCENT"]._options = _descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\322\260\326\300\001\002P1\232\320\206\375\001\001%'))
_UNITS_UNITCODE.values_by_name["NO_DIMENSION"].has_options = True
_UNITS_UNITCODE.values_by_name["NO_DIMENSION"]._options = _descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\322\260\326\300\001\003NDL'))
_UNITS_UNITCODE.values_by_name["PIXEL"].has_options = True
_UNITS_UNITCODE.values_by_name["PIXEL"]._options = _descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\322\260\326\300\001\002PX\232\320\206\375\001\002px'))
_UNITS_UNITCODE.values_by_name["PIXEL_LEVEL"].has_options = True
_UNITS_UNITCODE.values_by_name["PIXEL_LEVEL"]._options = _descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\322\260\326\300\001\003PXL'))
_UNITS_UNITCODE.values_by_name["ROTATIONS_PER_MINUTE"].has_options = True
_UNITS_UNITCODE.values_by_name["ROTATIONS_PER_MINUTE"]._options = _descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\322\260\326\300\001\003RPM\232\320\206\375\001\003rpm'))
_UNITS_UNITCODE.values_by_name["SECOND"].has_options = True
_UNITS_UNITCODE.values_by_name["SECOND"]._options = _descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\322\260\326\300\001\003SEC\232\320\206\375\001\001s'))
_UNITS_UNITCODE.values_by_name["MHZ"].has_options = True
_UNITS_UNITCODE.values_by_name["MHZ"]._options = _descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\322\260\326\300\001\003MHZ\232\320\206\375\001\003MHz'))
_UNITS_UNITCODE.values_by_name["HERTZ"].has_options = True
_UNITS_UNITCODE.values_by_name["HERTZ"]._options = _descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\322\260\326\300\001\003HTZ\232\320\206\375\001\002Hz'))
_UNITS_UNITCODE.values_by_name["MICROSECOND"].has_options = True
_UNITS_UNITCODE.values_by_name["MICROSECOND"]._options = _descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\322\260\326\300\001\003B98\232\320\206\375\001\003\302\265s'))
_UNITS_UNITCODE.values_by_name["MILLIMETER"].has_options = True
_UNITS_UNITCODE.values_by_name["MILLIMETER"]._options = _descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\322\260\326\300\001\003MMT\232\320\206\375\001\002mm'))
_UNITS_UNITCODE.values_by_name["CENTIMETER"].has_options = True
_UNITS_UNITCODE.values_by_name["CENTIMETER"]._options = _descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\322\260\326\300\001\002LC\232\320\206\375\001\002cm'))
_UNITS_UNITCODE.values_by_name["METER"].has_options = True
_UNITS_UNITCODE.values_by_name["METER"]._options = _descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\322\260\326\300\001\003MTR\232\320\206\375\001\001m'))
_UNITS_UNITCODE.values_by_name["PER_METER"].has_options = True
_UNITS_UNITCODE.values_by_name["PER_METER"]._options = _descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\322\260\326\300\001\003M0R\232\320\206\375\001\006m\342\201\273\302\271'))
_UNITS_UNITCODE.values_by_name["MILLILITER"].has_options = True
_UNITS_UNITCODE.values_by_name["MILLILITER"]._options = _descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\322\260\326\300\001\003MLT\232\320\206\375\001\002mL'))
_UNITS_UNITCODE.values_by_name["CUBIC_FOOT"].has_options = True
_UNITS_UNITCODE.values_by_name["CUBIC_FOOT"]._options = _descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\322\260\326\300\001\003MTQ\232\320\206\375\001\004Ft\302\263'))
_UNITS_UNITCODE.values_by_name["DECIBEL"].has_options = True
_UNITS_UNITCODE.values_by_name["DECIBEL"]._options = _descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\322\260\326\300\001\0022N\232\320\206\375\001\002dB'))
_UNITS_UNITCODE.values_by_name["DECIBEL_MW"].has_options = True
_UNITS_UNITCODE.values_by_name["DECIBEL_MW"]._options = _descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\322\260\326\300\001\0022N\232\320\206\375\001\004dBmW'))
_UNITS_UNITCODE.values_by_name["MICROAMP"].has_options = True
_UNITS_UNITCODE.values_by_name["MICROAMP"]._options = _descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\322\260\326\300\001\003B84\232\320\206\375\001\003\302\265A'))
_UNITS_UNITCODE.values_by_name["MILLIAMP"].has_options = True
_UNITS_UNITCODE.values_by_name["MILLIAMP"]._options = _descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\322\260\326\300\001\0024K\232\320\206\375\001\002mA'))
_UNITS_UNITCODE.values_by_name["MICROVOLT"].has_options = True
_UNITS_UNITCODE.values_by_name["MICROVOLT"]._options = _descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\322\260\326\300\001\003D82\232\320\206\375\001\003\302\265V'))
_UNITS_UNITCODE.values_by_name["VOLT"].has_options = True
_UNITS_UNITCODE.values_by_name["VOLT"]._options = _descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\322\260\326\300\001\003VLT\232\320\206\375\001\001V'))
_UNITS_UNITCODE.values_by_name["PICOFARAD"].has_options = True
_UNITS_UNITCODE.values_by_name["PICOFARAD"]._options = _descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\322\260\326\300\001\0024T\232\320\206\375\001\002pF'))
_UNITS_UNITCODE.values_by_name["COULOMB"].has_options = True
_UNITS_UNITCODE.values_by_name["COULOMB"]._options = _descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\322\260\326\300\001\003COU'))
_UNITS_UNITCODE.values_by_name["MILLIVOLT"].has_options = True
_UNITS_UNITCODE.values_by_name["MILLIVOLT"]._options = _descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\322\260\326\300\001\0022Z\232\320\206\375\001\002mV'))
_UNITS_UNITCODE.values_by_name["WATT"].has_options = True
_UNITS_UNITCODE.values_by_name["WATT"]._options = _descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\322\260\326\300\001\003WTT\232\320\206\375\001\001W'))
_UNITS_UNITCODE.values_by_name["AMPERE"].has_options = True
_UNITS_UNITCODE.values_by_name["AMPERE"]._options = _descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\322\260\326\300\001\003AMP\232\320\206\375\001\001A'))
_UNITS_UNITCODE.values_by_name["DEGREE_CELSIUS"].has_options = True
_UNITS_UNITCODE.values_by_name["DEGREE_CELSIUS"]._options = _descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\322\260\326\300\001\003CEL\232\320\206\375\001\003\302\260C'))
_UNITS_UNITCODE.values_by_name["KELVIN"].has_options = True
_UNITS_UNITCODE.values_by_name["KELVIN"]._options = _descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\322\260\326\300\001\003KEL\232\320\206\375\001\001K'))
_UNITS_UNITCODE.values_by_name["BYTE"].has_options = True
_UNITS_UNITCODE.values_by_name["BYTE"]._options = _descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\322\260\326\300\001\002AD'))
_UNITS_UNITCODE.values_by_name["MEGA_BYTES_PER_SECOND"].has_options = True
_UNITS_UNITCODE.values_by_name["MEGA_BYTES_PER_SECOND"]._options = _descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\322\260\326\300\001\004MBPS\232\320\206\375\001\004MB/s'))
_UNITS_UNITCODE.values_by_name["DEGREE"].has_options = True
_UNITS_UNITCODE.values_by_name["DEGREE"]._options = _descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\322\260\326\300\001\002DD\232\320\206\375\001\002\302\260'))
_UNITS_UNITCODE.values_by_name["RADIAN"].has_options = True
_UNITS_UNITCODE.values_by_name["RADIAN"]._options = _descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\322\260\326\300\001\003C81\232\320\206\375\001\003rad'))
_UNITS_UNITCODE.values_by_name["NEWTON"].has_options = True
_UNITS_UNITCODE.values_by_name["NEWTON"]._options = _descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\322\260\326\300\001\003NEW\232\320\206\375\001\001N'))
_UNITS_UNITCODE.values_by_name["CUBIC_CENTIMETER_PER_SEC"].has_options = True
_UNITS_UNITCODE.values_by_name["CUBIC_CENTIMETER_PER_SEC"]._options = _descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\322\260\326\300\001\0022J\232\320\206\375\001\006cm\302\263/s'))
_UNITS_UNITCODE.values_by_name["MILLIBAR"].has_options = True
_UNITS_UNITCODE.values_by_name["MILLIBAR"]._options = _descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\322\260\326\300\001\003MBR\232\320\206\375\001\004mbar'))
# @@protoc_insertion_point(module_scope)



from .core.test_descriptor import (
    Test
)

from .testplan.base import (
    TestPlan,
    TestSequence
)

from .standard import (
    EnvironmentType,
    get_env,
    is_development_env,
    is_production_env
)

from openhtf import *

from openhtf.util import conf



# Copyright 2014 Google Inc. All Rights Reserved.

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#     http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Modified by Tack Verification, 2020


"""Phases in OpenHTF.

Phases in OpenHTF are distinct steps in a test.  Each phase is an instance
of PhaseDescriptor class.

"""
import inspect
import pdb
import sys

import enum
import mutablerecords

import openhtf
from openhtf import util
from openhtf.core import test_record
import openhtf.plugs
from openhtf.util import data, functions

import six


class PhaseWrapError(Exception):
  """Error with phase wrapping."""


# Result of a phase.
#
# These values can be returned by a test phase to control what the framework
# does after the phase.
PhaseResult = enum.Enum('PhaseResult', [   # pylint: disable=invalid-name
    # Causes the framework to process the phase measurement outcomes and execute
    # the next phase.
    'CONTINUE',
    # Causes the framework to mark the phase with a fail outcome and execute the
    # next phase.
    'FAIL_AND_CONTINUE',
    # Causes the framework to execute the same phase again, ignoring the
    # measurement outcomes for this instance. If returned more than the phase's
    # repeat_limit option, this will be treated as a STOP.
    'REPEAT',
    # Causes the framework to ignore the measurement outcomes and execute the
    # next phase.  The phase is still logged, unlike with run_if.
    'SKIP',
    # Causes the framework to stop executing, indicating a failure.
    'STOP'
])

PhaseType = enum.Enum('PhaseType', [
  'MAIN',
  'SETUP',
  'TEARDOWN'
])


class PhaseOptions(mutablerecords.Record('PhaseOptions', [], {
    'name': None, 'timeout_s': None, 'run_if': None, 'requires_state': None,
    'repeat_limit': None, 'run_under_pdb': False})):
  """Options used to override default test phase behaviors.

  Attributes:
      name: Override for the name of the phase. Can be formatted in several
          different ways as defined in util.format_string.
      timeout_s: Timeout to use for the phase, in seconds.
      run_if: Callback that decides whether to run the phase or not; if not run,
          the phase will also not be logged. Optionally, this callback may take
          a single parameter: the test.state dictionnary. This allows dynamic
          test selection based on variables in the user defined state.
      requires_state: If True, pass the whole TestState into the first argument,
          otherwise only the TestApi will be passed in.  This is useful if a
          phase needs to wrap another phase for some reason, as
          PhaseDescriptors can only be invoked with a TestState instance.
      repeat_limit:  Maximum number of repeats.  None indicates a phase will
          be repeated infinitely as long as PhaseResult.REPEAT is returned.
      run_under_pdb: If True, run the phase under the Python Debugger (pdb).  When
          setting this option, increase the phase timeout as well because the
          timeout will still apply when under the debugger.

  Example Usages:
      @PhaseOptions(timeout_s=1)
      def PhaseFunc(test):
          pass

      @PhaseOptions(name='Phase({port})')
      def PhaseFunc(test, port, other_info):
          pass
  """

  def format_strings(self, **kwargs):
    """String substitution of name."""
    return mutablerecords.CopyRecord(
        self, name=util.format_string(self.name, kwargs))

  def update(self, **kwargs):
    for key, value in six.iteritems(kwargs):
      if key not in self.__slots__:
        raise AttributeError('Type %s does not have attribute %s' % (
            type(self).__name__, key))
      setattr(self, key, value)

  def __call__(self, phase_func):
    phase = PhaseDescriptor.wrap_or_copy(phase_func)
    for attr in self.__slots__:
      value = getattr(self, attr)
      if value is not None:
        setattr(phase.options, attr, value)
    return phase
  
  def call_run_if(self, user_defined_state):
    if self.run_if:
      arg_info = functions.getargspec(self.run_if)
      if arg_info.args:
        return self.run_if(user_defined_state)
      else:
        return self.run_if()
    else:
      return True
    

TestPhase = PhaseOptions


class PhaseDescriptor(mutablerecords.Record(
    'PhaseDescriptor', ['func'],
    {'options': PhaseOptions, 'plugs': list, 'measurements': list,
     'extra_kwargs': dict, 'code_info': test_record.CodeInfo.uncaptured(),
     'run_options': dict})):
  """Phase function and related information.

  Attributes:
    func: Function to be called (with TestApi as first argument).
    options: PhaseOptions instance.
    plugs: List of PhasePlug instances.
    measurements: List of Measurement objects.
    extra_kwargs: Keyword arguments that will be passed to the function.
    code_info: Info about the source code of func.
    type: PhaseType instance.
  """

  @classmethod
  def wrap_or_copy(cls, func, **options):
    """Return a new PhaseDescriptor from the given function or instance.

    We want to return a new copy so that you can reuse a phase with different
    options, plugs, measurements, etc.

    Args:
      func: A phase function or PhaseDescriptor instance.
      **options: Options to update on the result.

    Raises:
      PhaseWrapError: if func is a openhtf.PhaseGroup.

    Returns:
      A new PhaseDescriptor object.
    """
    if isinstance(func, openhtf.PhaseGroup):
      raise PhaseWrapError('Cannot wrap PhaseGroup <%s> as a phase.' % (
          func.name or 'Unnamed'))
    if isinstance(func, cls):
      # We want to copy so that a phase can be reused with different options
      # or kwargs.  See with_args() below for more details.
      retval = mutablerecords.CopyRecord(func)
    else:
      retval = cls(func)
    retval.options.update(**options)
    return retval
  
  def as_type(self, phase_type):
    self.run_options['type'] = phase_type
    return self
  
  def as_depth(self, depth):
    self.run_options['depth'] = depth
    return self

  def _asdict(self):
    asdict = {
        k: data.convert_to_base_types(getattr(self, k), ignore_keys=('cls',))
        for k in self.optional_attributes
    }
    asdict.update(name=self.name, doc=self.doc)
    return asdict

  @property
  def name(self):
    return self.options.name or self.func.__name__

  @property
  def doc(self):
    return self.func.__doc__

  def with_known_args(self, **kwargs):
    """Send only known keyword-arguments to the phase when called."""
    argspec = inspect.getfullargspec(self.func)
    stored = {}
    for key, arg in six.iteritems(kwargs):
      if argspec.varkw or key in argspec.args or key in argspec.kwonlyargs:
        stored[key] = arg
    if stored:
      return self.with_args(**stored)
    return self

  def with_args(self, **kwargs):
    """Send these keyword-arguments to the phase when called."""
    # Make a copy so we can have multiple of the same phase with different args
    # in the same test.
    new_info = mutablerecords.CopyRecord(self)
    new_info.options = new_info.options.format_strings(**kwargs)
    new_info.extra_kwargs.update(kwargs)
    new_info.measurements = [m.with_args(**kwargs) for m in self.measurements]
    return new_info

  def with_known_plugs(self, **subplugs):
    """Substitute only known plugs for placeholders for this phase."""
    return self._apply_with_plugs(subplugs, error_on_unknown=False)

  def with_plugs(self, **subplugs):
    """Substitute plugs for placeholders for this phase, error on unknowns."""
    return self._apply_with_plugs(subplugs, error_on_unknown=True)

  def _apply_with_plugs(self, subplugs, error_on_unknown):
    """Substitute plugs for placeholders for this phase.

    Args:
      subplugs: dict of plug name to plug class, plug classes to replace.
      error_on_unknown: bool, if True, then error when an unknown plug name is
          provided.

    Raises:
      openhtf.plugs.InvalidPlugError if for one of the plug names one of the
      following is true:
        - error_on_unknown is True and the plug name is not registered.
        - The new plug subclass is not a subclass of the original.
        - The original plug class is not a placeholder or automatic placeholder.

    Returns:
      PhaseDescriptor with updated plugs.
    """
    plugs_by_name = {plug.name: plug for plug in self.plugs}
    new_plugs = dict(plugs_by_name)

    for name, sub_class in six.iteritems(subplugs):
      original_plug = plugs_by_name.get(name)
      accept_substitute = True
      if original_plug is None:
        if not error_on_unknown:
          continue
        accept_substitute = False
      elif isinstance(original_plug.cls, openhtf.plugs.PlugPlaceholder):
        accept_substitute = issubclass(sub_class, original_plug.cls.base_class)
      else:
        # Check __dict__ to see if the attribute is explicitly defined in the
        # class, rather than being defined in a parent class.
        accept_substitute = ('auto_placeholder' in original_plug.cls.__dict__
                             and original_plug.cls.auto_placeholder
                             and issubclass(sub_class, original_plug.cls))

      if not accept_substitute:
        raise openhtf.plugs.InvalidPlugError(
            'Could not find valid placeholder for substitute plug %s '
            'required for phase %s' % (name, self.name))
      new_plugs[name] = mutablerecords.CopyRecord(original_plug, cls=sub_class)

    return mutablerecords.CopyRecord(
        self,
        plugs=list(new_plugs.values()),
        options=self.options.format_strings(**subplugs),
        measurements=[m.with_args(**subplugs) for m in self.measurements])

  def __call__(self, test_state):
    """Invoke this Phase, passing in the appropriate args.

    By default, an openhtf.TestApi is passed as the first positional arg, but if
    the 'requires_state' option is set, then a test_state.TestState is passed
    instead. If no positional args are expected, then neither is passed in. In
    any case, keyword args are passed in based on extra_kwargs, set via
    with_args(), combined with plugs (plugs override extra_kwargs).

    Args:
      test_state: test_state.TestState for the currently executing Test.

    Returns:
      The return value from calling the underlying function.
    """
    kwargs = dict(self.extra_kwargs)
    kwargs.update(test_state.plug_manager.provide_plugs(
        (plug.name, plug.cls) for plug in self.plugs if plug.update_kwargs))

    arg_info = functions.getargspec(self.func)
    keywords = arg_info.keywords
    # Pass in test_api if the phase takes *args, or **kwargs with at least 1
    # positional, or more positional args than we have keyword args.
    if arg_info.varargs or (keywords and len(arg_info.args) >= 1) or (
        len(arg_info.args) > len(kwargs)):
      args = []
      if self.options.requires_state:
        args.append(test_state)
      else:
        args.append(test_state.test_api)

      if self.options.run_under_pdb:
        return pdb.runcall(self.func, *args, **kwargs)
      else:
        return self.func(*args, **kwargs)
    if self.options.run_under_pdb:
      return pdb.runcall(self.func, **kwargs)
    else:
      return self.func(**kwargs)



# Copyright 2014 Google Inc. All Rights Reserved.

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#     http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


"""Implementation of the ADB SYNC protocol, used for push/pull commands.

This protocol is build on top of ADB streams, see adb_protocol.py for a high
level overview of this ADB protocol implementation. Note that we only implement
the host side of the protocol here, not the device side.

adb_cnxn = adb_protocol.AdbConnection.connect(usb_transport)
filesync = filesync_service.FilesyncService.using_connection(adb_cnxn)
stat = filesync.stat('/bin/sh')
print 'stat: ', stat

Documentation of this protocol is tough to find, so here it is:

Message Format:
  Every message starts with a single 32-bit word. (Everything is little-endian).
  Depending on that first word, the rest of the data can have various meanings.
  Messages from the host/desktop to the device always start with a 'request'
    message of a u32 command, u32 size, and size more bytes that's a filename,
    directory or symlink.
  The command here is referred to as 'id' in the code and the 4-letter codes
    are prefixed by ID_, eg ID_STAT.

  STAT:
    request:
      u32 command = 'STAT' == 0x53544154
      u32 size = len(filename) < 1024
      u8 data[size] = filename (no null)

    response:
      struct stat st = lstat(filename)
      u32 command = 'STAT'
      u32 mode = st.st_mode
      u32 size = st.st_size
      u32 time = st.st_mtime

  LIST:
    request:
      u32 command = 'LIST' == 0x4C495354
      u32 size = len(path) < 1024
      u8 data[size] = path (no null)

    response:
    for each filename in listing of path:
      struct stat st = lstat(filename)
      u32 command = 'DENT' == 0x44454E54
      u32 mode = st.st_mode
      u32 size = st.st_size
      u32 time = st.st_mtime
      u32 namelen = len(filename)
      u8 data[namelen] = filename

    done (device -> host):
      u32 command = 'DONE' == 0x444F4E45
      u32[4] = 0

  SEND:
    struct stat st = lstat(filename)
    request:
      fileinfo = sprintf(',%d', st.st_mode)
      u32 command = 'SEND' == 0x53454E44
      u32 size = len(filename) + len(fileinfo) < 1024
      u8 data[size] = filename + fileinfo

    repeated data command (host -> device):
      u32 command = 'DATA' == 0x44415441
      u32 size < (64 * 1024)
      u8 data[size] = file contents

    finish command (host -> device):
      u32 command = 'DONE' == 0x444F4E45
      u32 timestamp = st.st_mtime

    response (device -> host):
      u32 command = 'OKAY' == 0x4F4B4159 or 'FAIL' == 0x4641494C
      u32 size = 0 if 'OKAY' else len(fail_message)
      u8 data[] = fail_message

  RECV:
    request:
      u32 command = 'RECV' == 0x52454356
      u32 size = len(filename)
      u8 data[size] = filename

    repeated data response (device -> host):
      u32 command = 'DATA' == 0x44415441
      u32 size < (64 * 1024)
      u8 data[size] = file contents

    finish response (device -> host):
      u32 command = 'DONE' == 0x444F4E45
      u32 size = 0
"""

import collections
import stat
import struct
import sys
import time

from future.utils import raise_with_traceback
from openhtf.plugs.usb import adb_message
from openhtf.plugs.usb import usb_exceptions

# Default mode for pushed files, ADB will copy user permissions to group and
# other permissions, so we just default to full perms.
DEFAULT_PUSH_MODE = stat.S_IFREG | stat.S_IRWXU | stat.S_IRWXG | stat.S_IRWXO
# Maximum size of a filesync DATA packet.
MAX_PUSH_DATA_BYTES = 64 * 1024


DeviceFileStat = collections.namedtuple('DeviceFileStat', [
    'filename', 'mode', 'size', 'mtime'])


def _make_message_type(name, attributes, has_data=True):
  """Make a message type for the AdbTransport subclasses."""

  def assert_command_is(self, command):  # pylint: disable=invalid-name
    """Assert that a message's command matches the given command."""
    if self.command != command:
      raise usb_exceptions.AdbProtocolError(
          'Expected %s command, received %s', command, self)

  return type(name, (collections.namedtuple(name, attributes),),
              {
                  'assert_command_is': assert_command_is,
                  'has_data': has_data,
                  # Struct format on the wire has an unsigned int for each attr.
                  'struct_format': '<%sI' % len(attributes.split()),
              })


class FilesyncService(object):
  """Implements the FileSync protocol as described above.

  This class provides access to the methods provided by the ADB :sync protocol.
  Looking at /system/core/adb/file_sync_service.c in the Android source, the
  switch statement at the bottom of the file shows what commands are supported.

  Because the :sync service on the device side sits in a read-loop until the
  stream is closed, we can reuse the same stream for subsequent requests and
  save a little overhead instead of establishing a new stream each time, so we
  take an AdbStream on creation rather than an AdbConnection.  A helper
  classmethod, using_connection, is provided to create a FilesyncService
  directly using an AdbConnection.
  """

  def __init__(self, stream):
    self.stream = stream

  def __del__(self):  # pylint: disable=invalid-name
    self.close()

  def close(self):
    """Close the stream."""
    self.stream.close()

  def stat(self, filename, timeout=None):
    """Return device file stat."""
    transport = StatFilesyncTransport(self.stream)
    transport.write_data('STAT', filename, timeout)
    stat_msg = transport.read_message(timeout)
    stat_msg.assert_command_is('STAT')
    return DeviceFileStat(filename, stat_msg.mode, stat_msg.size, stat_msg.time)

  def list(self, path, timeout=None):
    """List directory contents on the device.

    Args:
      path: List the contents of this directory.
      timeout: Timeout to use for this operation.

    Returns:
      Generator yielding DeviceFileStat tuples representing the contents of
    the requested path.
    """
    transport = DentFilesyncTransport(self.stream)
    transport.write_data('LIST', path, timeout)
    return (DeviceFileStat(dent_msg.name, dent_msg.mode,
                           dent_msg.size, dent_msg.time) for dent_msg in
            transport.read_until_done('DENT', timeout))

  def recv(self, filename, dest_file, timeout=None):
    """Retrieve a file from the device into the file-like dest_file."""
    transport = DataFilesyncTransport(self.stream)
    transport.write_data('RECV', filename, timeout)
    for data_msg in transport.read_until_done('DATA', timeout):
      dest_file.write(data_msg.data)

  def _check_for_fail_message(self, transport, exc_info, timeout):  # pylint: disable=no-self-use
    """Check for a 'FAIL' message from transport.

    This method always raises, if 'FAIL' was read, it will raise an
    AdbRemoteError with the message, otherwise it will raise based on
    exc_info, which should be a tuple as per sys.exc_info().

    Args:
      transport: Transport from which to read for a 'FAIL' message.
      exc_info: Exception info to raise if no 'FAIL' is read.
      timeout: Timeout to use for the read operation.

    Raises:
      AdbRemoteError: If a 'FAIL' is read, otherwise raises exc_info.
    """
    try:
      transport.read_message(timeout)
    except usb_exceptions.CommonUsbError:
      # If we got a remote error, raise that exception.
      if sys.exc_info()[0] is usb_exceptions.AdbRemoteError:
        raise
    # Otherwise reraise the original exception.
    raise_with_traceback(exc_info[0](exc_info[1]), traceback=exc_info[2])

  # pylint: disable=too-many-arguments
  def send(self, src_file, filename, st_mode=DEFAULT_PUSH_MODE, mtime=None,
           timeout=None):
    """Push a file-like object to the device.

    Args:
      src_file: File-like object for reading from
      filename: Filename to push to on the device
      st_mode: stat mode for filename on the device
      mtime: modification time to set for the file on the device
      timeout: Timeout to use for the send operation.

    Raises:
      AdbProtocolError: If we get an unexpected response.
      AdbRemoteError: If there's a remote error (but valid protocol).
    """
    transport = DataFilesyncTransport(self.stream)
    transport.write_data('SEND', '%s,%s' % (filename, st_mode), timeout)

    try:
      while True:
        data = src_file.read(MAX_PUSH_DATA_BYTES)
        if not data:
          break
        transport.write_data('DATA', data, timeout)

      mtime = mtime or int(time.time())
      transport.write_message(
          FilesyncMessageTypes.DoneMessage('DONE', mtime), timeout)
    except usb_exceptions.AdbStreamClosedError:
      # Try to do one last read to see if we can get any more information,
      # ignoring any errors for this Read attempt. Note that this always
      # raises, either a new AdbRemoteError, or the AdbStreamClosedError.
      self._check_for_fail_message(transport, sys.exc_info(), timeout)

    data_msg = transport.read_message(timeout)
    data_msg.assert_command_is('OKAY')

  # pylint: enable=too-many-arguments

  @classmethod
  def using_connection(cls, connection, timeout=None):
    """Create a new FilesyncService using the given AdbConnection."""
    return cls(connection.open_stream('sync:', timeout))


class AbstractFilesyncTransport(object):
  """Transport to handle statefulness of the ADB sync service.

  Because message commands don't uniquely identify the structure of the message,
  this transport and filesync messages work a little differently than their ADB
  counterparts.  After an AdbStream is established with a remote :sync service,
  a 'req' message is sent to start a filesync transaction.  This transaction
  continues until a DONE message is sent, after which a new 'req' message may
  be sent (STAT commands only receive a single STAT response, and SEND commands
  receive an OKAY after sending a DONE message, so each command is actually a
  little different).

  The format of the messages transferred following the 'req' message depends on
  the id field of the 'req' message.  See this module's docstring for more
  details.  The FilesyncTransport exists between the sending of the 'req'
  message and the receiving/sending of the respective final message for the
  request type.  Upon creation of a FilesyncTransport, the caller determines
  which message type to expect by instantiating the appropriate subclass, based
  on the following union from ADB source:

    typedef union {
        unsigned id;
        struct {
            unsigned id;
            unsigned namelen;
        } req;
        struct {
            unsigned id;
            unsigned mode;
            unsigned size;
            unsigned time;
        } stat;
        struct {
            unsigned id;
            unsigned mode;
            unsigned size;
            unsigned time;
            unsigned namelen;
        } dent;
        struct {
            unsigned id;
            unsigned size;
        } data;
        struct {
            unsigned id;
            unsigned msglen;
        } status;
    } syncmsg;

  Subclasses are expected to provide the following class attributes:
    RECV_MSG_TYPE: The message type we expect to receive, must be one of the
      types in FilesyncMessageTypes below.
    VALID_RESPONSES: A tuple of valid command responses we can expect to
      receive over this transport.
  """
  CMD_TO_WIRE, WIRE_TO_CMD = adb_message.make_wire_commands(
      'STAT', 'LIST', 'SEND', 'RECV', 'DENT', 'DONE', 'DATA', 'OKAY', 'FAIL',
  )

  def __init__(self, stream):
    """Wrap the given stream in a FilesyncTransport.

    This transport takes ownership of the given stream - no other transports
    should use that stream until this one has been closed.  Closing this
    transport does *not* close the underlying stream, so it may be reused after
    this transport has been closed.  Note that transports provide no guarantee
    of thread-safety, unlike streams.  This means reads/writes are generally
    safe becauese of the thread safety provided by streamed, but Close() is not.

    This base class should not be instantiated directly - instead instantiate
    one of the subclasses below, based on what response type is expected.

    Args:
      stream: AdbStream to wrap with a new FilesyncTransport, it should be
        connected to the sync: service remotely.
    """
    assert hasattr(self, 'RECV_MSG_TYPE'), 'No RECV_MSG_TYPE set!'
    assert hasattr(self, 'VALID_RESPONSES'), 'No VALID_RESPONSES set!'
    self.stream = stream
    # pylint: disable=no-member

  def __str__(self):
    return '<%s(%s) id(%x), Receives: %s>' % (type(self).__name__, self.stream,
                                              id(self),
                                              self.RECV_MSG_TYPE.__name__)
  __repr__ = __str__

  def write_data(self, command, data, timeout=None):
    """Shortcut for writing specifically a DataMessage."""
    self.write_message(FilesyncMessageTypes.DataMessage(command, data), timeout)

  # pylint: disable=protected-access
  def write_message(self, msg, timeout=None):
    """Write an arbitrary message (of one of the types above).

    For the host side implementation, this will only ever be a DataMessage, but
    it's implemented generically enough here that you could use
    FilesyncTransport to implement the device side if you wanted.

    Args:
      msg:  The message to send, must be one of the types above.
      timeout: timeouts.PolledTimeout to use for the operation.
    """
    replace_dict = {'command': self.CMD_TO_WIRE[msg.command]}
    if msg.has_data:
      # Swap out data for the data length for the wire.
      data = msg[-1]
      replace_dict[msg._fields[-1]] = len(data)

    self.stream.write(struct.pack(msg.struct_format,
                                  *msg._replace(**replace_dict)), timeout)
    if msg.has_data:
      self.stream.write(data, timeout)

  # pylint: enable=protected-access

  def read_until_done(self, command, timeout=None):
    """Yield messages read until we receive a 'DONE' command.

    Read messages of the given command until we receive a 'DONE' command.  If a
    command different than the requested one is received, an AdbProtocolError
    is raised.

    Args:
      command: The command to expect, like 'DENT' or 'DATA'.
      timeout: The timeouts.PolledTimeout to use for this operation.

    Yields:
      Messages read, of type self.RECV_MSG_TYPE, see read_message().

    Raises:
      AdbProtocolError: If an unexpected command is read.
      AdbRemoteError: If a 'FAIL' message is read.
    """
    message = self.read_message(timeout)
    while message.command != 'DONE':
      message.assert_command_is(command)
      yield message
      message = self.read_message(timeout)

  def read_message(self, timeout=None):
    """Read a message from this transport and return it.

    Reads a message of RECV_MSG_TYPE and returns it.  Note that this method
    abstracts the data length and data read so that the caller simply gets the
    data along with the header in the returned message.

    Args:
      timeout: timeouts.PolledTimeout to use for the operation.

    Returns:
      An instance of self.RECV_MSG_TYPE that was read from self.stream.

    Raises:
      AdbProtocolError: If an invalid response is received.
      AdbRemoteError: If a FAIL response is received.
    """
    raw_data = self.stream.read(
        struct.calcsize(self.RECV_MSG_TYPE.struct_format), timeout)
    try:
      raw_message = struct.unpack(self.RECV_MSG_TYPE.struct_format, raw_data)
    except struct.error:
      raise usb_exceptions.AdbProtocolError(
          '%s expected format "%s", got data %s', self,
          self.RECV_MSG_TYPE.struct_format, raw_data)

    if raw_message[0] not in self.WIRE_TO_CMD:
      raise usb_exceptions.AdbProtocolError(
          'Unrecognized command id: %s', raw_message)

    # Swap out the wire command with the string equivalent.
    raw_message = (self.WIRE_TO_CMD[raw_message[0]],) + raw_message[1:]

    if self.RECV_MSG_TYPE.has_data and raw_message[-1]:
      # For messages that have data, the length of the data is the last field
      # in the struct.  We do another read and swap out that length for the
      # actual data read before we create the namedtuple to return.
      data_len = raw_message[-1]
      raw_message = raw_message[:-1] + (self.stream.read(data_len, timeout),)

    if raw_message[0] not in self.VALID_RESPONSES:
      raise usb_exceptions.AdbProtocolError(
          '%s not a valid response for %s', raw_message[0], self)
    if raw_message[0] == 'FAIL':
      raise usb_exceptions.AdbRemoteError(
          'Remote ADB failure: %s', raw_message)
    return self.RECV_MSG_TYPE(*raw_message)


class FilesyncMessageTypes(object):
  """Container for the various message types used by the Filesync protocol.

  These message types correspond roughly to the struct types contained within
  the union above.  Note that which commands are valid for which message types
  varies, see the header at the top of this file for details.  Also, we use
  DataMessage for status and req structs because they have the same format. We
  create a new type 'DoneMessage' because ADB uses a data message *with no
  actual data* to terminate a SEND command - and uses the 'size' field to send
  the mtime of the file... derp, this is actually a different type entirely, so
  we do that.
  """
  # pylint: disable=invalid-name
  DoneMessage = _make_message_type('DoneMessage',
                                   'command mtime',
                                   has_data=False)
  StatMessage = _make_message_type('StatMessage', 'command mode size time',
                                   has_data=False)
  DentMessage = _make_message_type('DentMessage', 'command mode size time name')
  DataMessage = _make_message_type('DataMessage', 'command data')
  # pylint: enable=invalid-name


class StatFilesyncTransport(AbstractFilesyncTransport):
  """Subclass of AbstractFilesyncTransport for STAT requests."""
  RECV_MSG_TYPE = FilesyncMessageTypes.StatMessage
  VALID_RESPONSES = ('STAT',)


class DentFilesyncTransport(AbstractFilesyncTransport):
  """Subclass of AbstractFilesyncTransport for LIST requests."""
  RECV_MSG_TYPE = FilesyncMessageTypes.DentMessage
  VALID_RESPONSES = ('DENT', 'DONE')


class DataFilesyncTransport(AbstractFilesyncTransport):
  """Sublcass of AbstractFilesyncTransport for DATA-based requests."""
  RECV_MSG_TYPE = FilesyncMessageTypes.DataMessage
  VALID_RESPONSES = ('DATA', 'DONE', 'OKAY', 'FAIL')



# Copyright 2014 Google Inc. All Rights Reserved.

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#     http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Modified by Tack Verification, 2020


"""Measurements for OpenHTF.

Measurements in OpenHTF are used to represent values collected during a Test.
They can be numeric or string values, and can be configured such that the
OpenHTF framework will automatically check them against Pass/Fail criteria.
Measurements should not be used for large binary blobs, which are instead best
stored as Attachments (see attachments.py).

Measurements are described by the measurements.Measurement class.  Essentially,
the Measurement class is used by test authors to declare measurements by name,
and to optionally provide unit, type, and validation information.  Measurements
are attached to Test Phases using the @measurements.measures() decorator.

When measurements are output by the OpenHTF framework, the Measurement objects
are serialized into the 'measurements' field on the PhaseRecord, which contain
both descriptor fields, outcome (PASS/FAIL/UNSET), and the values themselves.

Validation of undimensioned measurements happens when they are set, so that
users of the HTTP API can see PASS/FAIL outcome on those measurements
immediately after they are set.  Multidimensional measurements, however,
don't usually make sense to validate until all data is available, so they
instead enter a PARTIALLY_SET outcome state until the end of the test phase,
at which point they are validated and become with PASS or FAIL.  Note that
validators of dimensioned measurements are only called at the end of the phase
if at least one value was set in the multidimensional measurement, otherwise it
remains UNSET, so that outcome fields for all measurements may be PASS, FAIL,
or UNSET.

# TODO(madsci): Make validators.py example.
See examples/validators.py for some examples on how to define and use custom
measurement validators.

Examples:

  @measurements.measures(
      measurements.Measurement(
          'number_widgets').in_range(5, 10).doc(
          '''This phase parameter tracks the number of widgets.'''))
  @measurements.measures(
      *(measurements.Measurement('level_%s' % lvl)
        for lvl in ('none', 'some', 'all')))
  def WidgetTestPhase(test):
    test.measurements.number_widgets = 5
    test.measurements.level_none = 10

"""


import collections
import logging

from enum import Enum

import mutablerecords

from openhtf import util
from openhtf.core import phase_descriptor
from openhtf.util import data
from openhtf.util import validators
from openhtf.util import units
import six

try:
  import pandas
except ImportError:
  pandas = None

_LOG = logging.getLogger(__name__)


class InvalidDimensionsError(Exception):
  """Raised when there is a problem with measurement dimensions."""


class InvalidMeasurementType(Exception):
  """Raised when an unexpected measurement type is given."""


class MeasurementNotSetError(Exception):
  """Raised when a measurement is accessed that hasn't been set."""


class NotAMeasurementError(Exception):
  """Raised if an invalid measurement name is accessed."""


class DuplicateNameError(Exception):
  """An exception which occurs when a measurement name collision occurs."""


# Only multidimensional measurements can be 'PARTIALLY_SET', and can never be in
# that state after their respective phase has completed (they must transition to
# either PASS or FAIL at that point).
Outcome = Enum('Outcome', ['PASS', 'FAIL', 'UNSET', 'PARTIALLY_SET'])


class Measurement(  # pylint: disable=no-init
    mutablerecords.Record(
        'Measurement', ['name'],
        {'units': None, 'dimensions': None, 'docstring': None,
         '_notification_cb': None,
         'validators': list,
         'outcome': Outcome.UNSET,
         'measured_value': None,
         '_cached': None})):
  """Record encapsulating descriptive data for a measurement.

  This record includes an _asdict() method so it can be easily output.  Output
  is as you would expect, a dict mapping non-None fields to their values
  (validators are stringified with str()).

  Attributes:
    name: Name of the measurement.
    docstring: Optional string describing this measurement.
    units: UOM code of the units for the measurement being taken.
    dimensions: Tuple of UOM codes for units of dimensions.
    validators: List of callable validator objects to perform pass/fail checks.
    outcome: One of the Outcome() enumeration values, starting at UNSET.
    measured_value: An instance of MeasuredValue or DimensionedMeasuredValue
      containing the value(s) of this Measurement that have been set, if any.
    _cached: A cached dict representation of this measurement created initially
      during as_base_types and updated in place to save allocation time.
  """

  def __init__(self, name, **kwargs):
    super(Measurement, self).__init__(name, **kwargs)
    if 'measured_value' not in kwargs:
      self._initialize_value()

  def _initialize_value(self):
    if self.measured_value and self.measured_value.is_value_set:
      raise ValueError('Cannot update a Measurement once a value is set.')

    if self.dimensions:
      self.measured_value = DimensionedMeasuredValue(
          self.name, len(self.dimensions))
    else:
      self.measured_value = MeasuredValue(self.name)

  def __setattr__(self, attr, value):
    super(Measurement, self).__setattr__(attr, value)
    # When dimensions changes, we may need to update our measured_value type.
    if attr == 'dimensions':
      self._initialize_value()

  def __setstate__(self, state):
    """Set this record's state during unpickling.

    This override is necessary to ensure that the the _initialize_value check
    is skipped during unpickling.
    """
    dimensions = state.pop('dimensions')
    super(Measurement, self).__setstate__(state)
    object.__setattr__(self, 'dimensions', dimensions)

  def set_notification_callback(self, notification_cb):
    """Set the notifier we'll call when measurements are set."""
    self._notification_cb = notification_cb
    if not notification_cb and self.dimensions:
      self.measured_value.notify_value_set = None
    return self

  def notify_value_set(self):
    if self.dimensions:
      self.outcome = Outcome.PARTIALLY_SET
    else:
      self.validate()
    if self._notification_cb:
      self._notification_cb()

  def doc(self, docstring):
    """Set this Measurement's docstring, returns self for chaining."""
    self.docstring = docstring
    return self

  def _maybe_make_unit_desc(self, unit_desc):
    """Return the UnitDescriptor or convert a string to one."""
    if isinstance(unit_desc, str) or unit_desc is None:
      unit_desc = units.Unit(unit_desc)
    if not isinstance(unit_desc, units.UnitDescriptor):
      raise TypeError('Invalid units for measurement %s: %s' % (self.name,
                                                                unit_desc))
    return unit_desc

  def _maybe_make_dimension(self, dimension):
    """Return a `measurements.Dimension` instance."""
    # For backwards compatibility the argument can be either a Dimension, a
    # string or a `units.UnitDescriptor`.
    if isinstance(dimension, Dimension):
      return dimension
    if isinstance(dimension, units.UnitDescriptor):
      return Dimension.from_unit_descriptor(dimension)
    if isinstance(dimension, str):
      return Dimension.from_string(dimension)

    raise TypeError('Cannot convert %s to a dimension', dimension)

  def with_units(self, unit_desc):
    """Declare the units for this Measurement, returns self for chaining."""
    self.units = self._maybe_make_unit_desc(unit_desc)
    return self

  def with_dimensions(self, *dimensions):
    """Declare dimensions for this Measurement, returns self for chaining."""
    self.dimensions = tuple(
        self._maybe_make_dimension(dim) for dim in dimensions)
    self._cached = None
    return self

  def with_validator(self, validator):
    """Add a validator callback to this Measurement, chainable."""
    if not callable(validator):
      raise ValueError('Validator must be callable', validator)
    self.validators.append(validator)
    self._cached = None
    return self

  def with_args(self, **kwargs):
    """String substitution for names and docstrings."""
    validators = [
        validator.with_args(**kwargs)
        if hasattr(validator, 'with_args') else validator
        for validator in self.validators
    ]
    return mutablerecords.CopyRecord(
        self, name=util.format_string(self.name, kwargs),
        docstring=util.format_string(self.docstring, kwargs),
        validators=validators,
        _cached=None,
    )

  def __getattr__(self, attr):  # pylint: disable=invalid-name
    """Support our default set of validators as direct attributes."""
    # Don't provide a back door to validators.py private stuff accidentally.
    if attr.startswith('_') or not validators.has_validator(attr):
      raise AttributeError("'%s' object has no attribute '%s'" % (
          type(self).__name__, attr))

    # Create a wrapper to invoke the attribute from within validators.
    def _with_validator(*args, **kwargs):  # pylint: disable=invalid-name
      return self.with_validator(
          validators.create_validator(attr, *args, **kwargs))
    return _with_validator

  def validate(self):
    """Validate this measurement and update its 'outcome' field."""
    # PASS if all our validators return True, otherwise FAIL.
    try:
      if all(v(self.measured_value.value) for v in self.validators):
        self.outcome = Outcome.PASS
      else:
        self.outcome = Outcome.FAIL
      return self
    except Exception as e:  # pylint: disable=bare-except
      _LOG.error('Validation for measurement %s raised an exception %s.',
                 self.name, e)
      self.outcome = Outcome.FAIL
      raise
    finally:
      if self._cached:
        self._cached['outcome'] = self.outcome.name

  def as_base_types(self):
    """Convert this measurement to a dict of basic types."""
    if not self._cached:
      # Create the single cache file the first time this is called.
      self._cached = {
          'name': self.name,
          'outcome': self.outcome.name,
      }
      if self.validators:
        self._cached['validators'] = data.convert_to_base_types(
            tuple(str(v) for v in self.validators))
      if self.dimensions:
        self._cached['dimensions'] = data.convert_to_base_types(self.dimensions)
      if self.units:
        self._cached['units'] = data.convert_to_base_types(self.units)
      if self.docstring:
        self._cached['docstring'] = self.docstring
    if self.measured_value.is_value_set:
      self._cached['measured_value'] = self.measured_value.basetype_value()
    return self._cached

  def to_dataframe(self, columns=None):
    """Convert a multi-dim to a pandas dataframe."""
    if not isinstance(self.measured_value, DimensionedMeasuredValue):
      raise TypeError(
        'Only a dimensioned measurement can be converted to a DataFrame')

    if columns is None:
      columns = [d.name for d in self.dimensions]
      columns += [self.units.name if self.units else 'value']

    dataframe = self.measured_value.to_dataframe(columns)

    return dataframe


class MeasuredValue(
    mutablerecords.Record('MeasuredValue', ['name'],
                          {'stored_value': None, 'is_value_set': False,
                           '_cached_value': None})):
  """Class encapsulating actual values measured.

  Note that this is really just a value wrapper with some sanity checks.  See
  Declaration for the class that handles the descriptive aspect of the
  measurement.  This class is the type that Collection actually stores in
  its _values attribute.

  This class stores values for un-dimensioned (single-value) measurements, for
  dimensioned values, see the DimensionedMeasuredValue.  The interfaces are very
  similar, but differ slightly; the important part is the get_value() interface
  on both of them.

  The _cached_value is the base type represention of the stored_value when that
  is set.
  """

  def __str__(self):
    return str(self.value) if self.is_value_set else 'UNSET'

  def __eq__(self, other):
    return (type(self) == type(other) and self.name == other.name and
            self.is_value_set == other.is_value_set and
            self.stored_value == other.stored_value)

  def __ne__(self, other):
    return not self.__eq__(other)

  @property
  def value(self):
    if not self.is_value_set:
      raise MeasurementNotSetError('Measurement not yet set', self.name)
    return self.stored_value

  def basetype_value(self):
    return self._cached_value

  def set(self, value):
    """Set the value for this measurement, with some sanity checks."""
    if self.is_value_set:
      # While we want to *allow* re-setting previously set measurements, we'd
      # rather promote the use of multidimensional measurements instead of
      # discarding data, so we make this somewhat chatty.
      _LOG.warning(
          'Overriding previous measurement %s value of %s with %s, the old '
          'value will be lost.  Use a dimensioned measurement if you need to '
          'save multiple values.', self.name, self.stored_value, value)
    if value is None:
      _LOG.warning('Measurement %s is set to None', self.name)
    self.stored_value = value
    self._cached_value = data.convert_to_base_types(value)
    self.is_value_set = True


class Dimension(object):
  """Dimension for multi-dim Measurements.

  Dimensions optionally include a unit and a description.  This is intended
  as a drop-in replacement for UnitDescriptor for backwards compatibility.
  """

  __slots__ = ['_description', '_unit', '_cached_dict']

  def __init__(self, description='', unit=units.NO_DIMENSION):
    self._description = description
    self._unit = unit
    self._cached_dict = data.convert_to_base_types({
        'code': self.code,
        'description': self.description,
        'name': self.name,
        'suffix': self.suffix,
    })

  def __eq__(self, other):
    return (self.description == other.description and self.unit == other.unit)

  def __ne__(self, other):
    return not self == other

  def __repr__(self):
    return '<%s: %s>' % (type(self).__name__, self._asdict())

  @classmethod
  def from_unit_descriptor(cls, unit_desc):
    return cls(unit=unit_desc)

  @classmethod
  def from_string(cls, string):
    """Convert a string into a Dimension"""
    # Note: There is some ambiguity as to whether the string passed is intended
    # to become a unit looked up by name or suffix, or a Dimension descriptor.
    if string in units.UNITS_BY_ALL:
      return cls(description=string, unit=units.Unit(string))
    else:
      return cls(description=string)

  @property
  def description(self):
    return self._description

  @property
  def unit(self):
    return self._unit

  @property
  def code(self):
    """Provides backwards compatibility to `units.UnitDescriptor` api."""
    return self._unit.code

  @property
  def suffix(self):
    """Provides backwards compatibility to `units.UnitDescriptor` api."""
    return self._unit.suffix

  @property
  def name(self):
    """Provides backwards compatibility to `units.UnitDescriptor` api."""
    return self._description or self._unit.name

  def _asdict(self):
    return self._cached_dict


class DimensionedMeasuredValue(mutablerecords.Record(
    'DimensionedMeasuredValue', ['name', 'num_dimensions'],
    {'notify_value_set': None,
     'value_dict': collections.OrderedDict,
     '_cached_basetype_values': list})):
  """Class encapsulating actual values measured.

  See the MeasuredValue class docstring for more info.  This class provides a
  dict-like interface for indexing into dimensioned measurements.

  The _cached_basetype_values is a cached list of the dimensioned entries in
  order of being set.  Each list entry is a tuple that is composed of the key,
  then the value.  This is set to None if a previous measurement is overridden;
  in such a case, the list is fully reconstructed on the next call to
  basetype_value.
  """
  def __str__(self):
    return str(self.value) if self.is_value_set else 'UNSET'

  def with_notify(self, notify_value_set):
    self.notify_value_set = notify_value_set
    return self

  @property
  def is_value_set(self):
    return len(self.value_dict) > 0

  def __iter__(self):  # pylint: disable=invalid-name
    """Iterate over items, allows easy conversion to a dict."""
    return iter(six.iteritems(self.value_dict))

  def __setitem__(self, coordinates, value):  # pylint: disable=invalid-name
    coordinates_len = len(coordinates) if hasattr(coordinates, '__len__') else 1
    if coordinates_len != self.num_dimensions:
      raise InvalidDimensionsError(
          'Expected %s-dimensional coordinates, got %s' % (self.num_dimensions,
                                                           coordinates_len))

    # Wrap single dimensions in a tuple so we can assume value_dict keys are
    # always tuples later.
    if self.num_dimensions == 1:
      coordinates = (coordinates,)

    if coordinates in self.value_dict:
      _LOG.warning(
          'Overriding previous measurement %s[%s] value of %s with %s',
          self.name, coordinates, self.value_dict[coordinates], value)
      self._cached_basetype_values = None
    elif self._cached_basetype_values is not None:
      self._cached_basetype_values.append(data.convert_to_base_types(
          coordinates + (value,)))

    self.value_dict[coordinates] = value

    if self.notify_value_set:
      self.notify_value_set()

  def __getitem__(self, coordinates):  # pylint: disable=invalid-name
    # Wrap single dimensions in a tuple so we can assume value_dict keys are
    # always tuples later.
    if self.num_dimensions == 1:
      coordinates = (coordinates,)
    return self.value_dict[coordinates]

  @property
  def value(self):
    """The values stored in this record.

    Returns:
      A list of tuples; the last element of each tuple will be the measured
      value, the other elements will be the associated coordinates.  The tuples
      are output in the order in which they were set.
    """
    if not self.is_value_set:
      raise MeasurementNotSetError('Measurement not yet set', self.name)
    return [dimensions + (value,) for dimensions, value in
            six.iteritems(self.value_dict)]

  def basetype_value(self):
    if self._cached_basetype_values is None:
      self._cached_basetype_values = list(
          data.convert_to_base_types(coordinates + (value,))
          for coordinates, value in six.iteritems(self.value_dict))
    return self._cached_basetype_values

  def to_dataframe(self, columns=None):
    """Converts to a `pandas.DataFrame`"""
    if not self.is_value_set:
      raise ValueError('Value must be set before converting to a DataFrame.')
    if not pandas:
      raise RuntimeError('Install pandas to convert to pandas.DataFrame')
    return pandas.DataFrame.from_records(self.value, columns=columns)



class Collection(mutablerecords.Record('Collection', ['_measurements'])):
  """Encapsulates a collection of measurements.

  This collection can have measurement values retrieved and set via getters and
  setters that provide attribute and dict-like interfaces.

  A Collection is created with a list of Measurement objects (defined above).
  Measurements can't be added after initialization, only accessed and set.

  MeasuredValue values can be set as attributes (see below).  They can also be
  read as attributes, but you get a DimensionedMeasuredValue object back if the
  measurement accessed is dimensioned (this is how setting of dimensioned
  measurements works, and so is unavoidable).

  Iterating over a Collection results in (key, value) tuples of only set
  measurements and their values.  As such, a Collection can be converted to
  a dict if you want to see all of a dimensioned measurement's values.
  Alternatively, DimensionedMeasuredValue objects can also be converted directly
  to dicts with dict(), as they also support an __iter__() interface.

  This class is intended for use only internally within the OpenHTF framework.

  Example:
    from openhtf.util import measurements
    from openhtf.util.units import UOM

    self.measurements = measurements.Collection([
        measurements.Measurement('widget_height'),
        measurements.Measurement('widget_freq_response').with_dimensions(
            UOM['HERTZ'])])
    self.measurements.widget_height = 3
    print self.measurements.widget_height            # 3
    self.measurements.widget_freq_response[5] = 10
    print self.measurements.widget_freq_response[5]  # 10
    self.measurements.widget_freq_response[6] = 11
    print dict(self.measurements.widget_freq_response)
    # {5: 10, 6: 11}

    # Not recommended, but you can also do this.  This is intended only for
    # framework internal use when generating the output test record.
    print dict(self.measurements)['widget_freq_response']
    # [(5, 10), (6, 11)]
  """

  def _assert_valid_key(self, name):
    """Raises if name is not a valid measurement."""
    if name not in self._measurements:
      raise NotAMeasurementError('Not a measurement', name, self._measurements)

  def __iter__(self):  # pylint: disable=invalid-name
    """Extract each MeasurementValue's value."""
    return ((key, meas.measured_value.value)
            for key, meas in six.iteritems(self._measurements))

  def __setattr__(self, name, value):  # pylint: disable=invalid-name
    self[name] = value

  def __getattr__(self, name):  # pylint: disable=invalid-name
    return self[name]

  def __setitem__(self, name, value):  # pylint: disable=invalid-name
    self._assert_valid_key(name)
    if self._measurements[name].dimensions:
      raise InvalidDimensionsError(
          'Cannot set dimensioned measurement without indices')
    self._measurements[name].measured_value.set(value)
    self._measurements[name].notify_value_set()

  def __getitem__(self, name):  # pylint: disable=invalid-name
    self._assert_valid_key(name)

    if self._measurements[name].dimensions:
      return self._measurements[name].measured_value.with_notify(
          self._measurements[name].notify_value_set)

    # Return the MeasuredValue's value, MeasuredValue will raise if not set.
    return self._measurements[name].measured_value.value


def measures(*measurements, **kwargs):
  """Decorator-maker used to declare measurements for phases.

  See the measurements module docstring for examples of usage.

  Args:
    measurements: Measurement objects to declare, or a string name from which
        to create a Measurement.
    kwargs: Keyword arguments to pass to Measurement constructor if we're
        constructing one.  Note that if kwargs are provided, the length
        of measurements must be 1, and that value must be a string containing
        the measurement name.  For valid kwargs, see the definition of the
        Measurement class.

  Returns:
    A decorator that declares the measurement(s) for the decorated phase.
  """
  def _maybe_make(meas):
    """Turn strings into Measurement objects if necessary."""
    if isinstance(meas, Measurement):
      return meas
    elif isinstance(meas, six.string_types):
      return Measurement(meas, **kwargs)
    raise InvalidMeasurementType('Expected Measurement or string', meas)

  # In case we're declaring a measurement inline, we can only declare one.
  if kwargs and len(measurements) != 1:
    raise InvalidMeasurementType(
        'If @measures kwargs are provided, a single measurement name must be '
        'provided as a positional arg first.')

  # Unlikely, but let's make sure we don't allow overriding initial outcome.
  if 'outcome' in kwargs:
    raise ValueError('Cannot specify outcome in measurement declaration!')

  measurements = [_maybe_make(meas) for meas in measurements]

  # 'measurements' is guaranteed to be a list of Measurement objects here.
  def decorate(wrapped_phase):
    """Phase decorator to be returned."""
    phase = phase_descriptor.PhaseDescriptor.wrap_or_copy(wrapped_phase)
    duplicate_names = (set(m.name for m in measurements) &
                       set(m.name for m in phase.measurements))
    if duplicate_names:
      raise DuplicateNameError('Measurement names duplicated', duplicate_names)

    phase.measurements.extend(measurements)
    return phase
  return decorate



import os

from openhtf import util
from openhtf.util import data

from openhtf.core.test_record import Outcome
from openhtf.output.callbacks.json_factory import OutputToJSON


class LocalStorageOutput(OutputToJSON):
    def __init__(self, folder_pattern, indent=4):
        self.folder_pattern = folder_pattern
        super().__init__(indent=indent)

    def __call__(self, test_record):
        if test_record.is_started():
            record_dict = data.convert_to_base_types(
                test_record, ignore_keys=('code_info', 'phases', 'log_records'))

            output_folder = util.format_string(self.folder_pattern, record_dict)

            if not os.path.exists(output_folder):
                os.makedirs(output_folder)

            with open(os.path.join(output_folder, 'result.json'), 'w+') as outfile:
                outfile.write(self.serialize_test_record(test_record))
            
            for phase in test_record.phases:
                for attachment_name, attachment in phase.attachments.items():
                    with open(os.path.join(output_folder, attachment_name), 'wb+') as outfile:
                        outfile.write(attachment.data)




# Copyright 2014 Google Inc. All Rights Reserved.

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#     http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


"""Plugs that provide access to USB devices via ADB/Fastboot.

For details of what these interfaces look like, see adb_device.py and
fastboot_device.py.

To use these plugs:
  from openhtf import plugs
  from openhtf.plugs import usb

  @plugs.RequiresPlug(adb=usb.AdbPlug)
  def MyPhase(test, adb):
    adb.Shell('ls')
"""
import argparse
import logging
import time

import openhtf.plugs as plugs
from openhtf.plugs import cambrionix
from openhtf.plugs.usb import adb_device
from openhtf.plugs.usb import adb_protocol
from openhtf.plugs.usb import fastboot_device
from openhtf.plugs.usb import fastboot_protocol
from openhtf.plugs.usb import local_usb
from openhtf.plugs.usb import usb_exceptions
from openhtf.plugs.user_input import prompt_for_test_start
from openhtf.util import conf
from openhtf.util import functions

_LOG = logging.getLogger(__name__)

conf.declare('libusb_rsa_key', 'A private key file for use by libusb auth.')
conf.declare('remote_usb', 'ethersync or other')
conf.declare('ethersync', 'ethersync configuration')


@functions.call_once
def init_dependent_flags():
  parser = argparse.ArgumentParser(
      'USB Plug flags', parents=[
          adb_protocol.ARG_PARSER, fastboot_protocol.ARG_PARSER],
      add_help=False)
  parser.parse_known_args()


def _open_usb_handle(serial_number=None, **kwargs):
  """Open a UsbHandle subclass, based on configuration.

  If configuration 'remote_usb' is set, use it to connect to remote usb,
  otherwise attempt to connect locally.'remote_usb' is set to usb type,
  EtherSync or other.

  Example of Cambrionix unit in config:
  remote_usb: ethersync
  ethersync:
       mac_addr: 78:a5:04:ca:91:66
       plug_port: 5

  Args:
    serial_number: Optional serial number to connect to.
    **kwargs: Arguments to pass to respective handle's Open() method.

  Returns:
    Instance of UsbHandle.
  """
  init_dependent_flags()
  remote_usb = conf.remote_usb
  if remote_usb:
    if remote_usb.strip() == 'ethersync':
      device = conf.ethersync
      try:
        mac_addr = device['mac_addr']
        port = device['plug_port']
      except (KeyError, TypeError):
        raise ValueError('Ethersync needs mac_addr and plug_port to be set')
      else:
        ethersync = cambrionix.EtherSync(mac_addr)
        serial_number = ethersync.get_usb_serial(port)

  return local_usb.LibUsbHandle.open(serial_number=serial_number, **kwargs)


class FastbootPlug(plugs.BasePlug):
  """Plug that provides fastboot."""

  def __init__(self):
    self._device = fastboot_device.FastbootDevice.connect(
        _open_usb_handle(
            interface_class=fastboot_device.CLASS,
            interface_subclass=fastboot_device.SUBCLASS,
            interface_protocol=fastboot_device.PROTOCOL))

  def tearDown(self):
    self._device.close()

  def __getattr__(self, attr):
    """Forward other attributes to the device."""
    return getattr(self._device, attr)


class AdbPlug(plugs.BasePlug):
  """Plug that provides ADB."""

  serial_number = None

  def __init__(self):
    if conf.libusb_rsa_key:
      self._rsa_keys = [adb_device.M2CryptoSigner(conf.libusb_rsa_key)]
    else:
      self._rsa_keys = None
    self._device = None
    self.connect()

  def tearDown(self):
    if self._device:
      self._device.close()

  def connect(self):
    if self._device:
      try:
        self._device.close()
      except (usb_exceptions.UsbWriteFailedError,
              usb_exceptions.UsbReadFailedError):
        pass
      self._device = None

    kwargs = {}
    if self._rsa_keys:
      kwargs['rsa_keys'] = self._rsa_keys

    self._device = adb_device.AdbDevice.connect(
        _open_usb_handle(
            interface_class=adb_device.CLASS,
            interface_subclass=adb_device.SUBCLASS,
            interface_protocol=adb_device.PROTOCOL,
            serial_number=self.serial_number),
        **kwargs)

  def __getattr__(self, attr):
    """Forward other attributes to the device."""
    return getattr(self._device, attr)


class AndroidTriggers(object):  # pylint: disable=invalid-name
  """Test start and stop triggers for Android devices."""

  @classmethod
  def _try_open(cls):
    """Try to open a USB handle."""
    handle = None
    for usb_cls, subcls, protocol in [(adb_device.CLASS,
                                       adb_device.SUBCLASS,
                                       adb_device.PROTOCOL),
                                      (fastboot_device.CLASS,
                                       fastboot_device.SUBCLASS,
                                       fastboot_device.PROTOCOL)]:
      try:
        handle = local_usb.LibUsbHandle.open(
            serial_number=cls.serial_number,
            interface_class=usb_cls,
            interface_subclass=subcls,
            interface_protocol=protocol)
        cls.serial_number = handle.serial_number
        return True
      except usb_exceptions.DeviceNotFoundError:
        pass
      except usb_exceptions.MultipleInterfacesFoundError:
        _LOG.warning('Multiple Android devices found, ignoring!')
      finally:
        if handle:
          handle.close()
    return False

  @classmethod
  def test_start_frontend(cls):
    """Start when frontend event comes, but get serial from USB."""
    prompt_for_test_start('Connect Android device and press ENTER.',
                          text_input=False)()
    return cls.test_start()

  @classmethod
  def test_start(cls):
    """Returns serial when the test is ready to start."""
    while not cls._try_open():
      time.sleep(1)
    return cls.serial_number

  @classmethod
  def test_stop(cls):
    """Returns True when the test is completed and can restart."""
    while cls._try_open():
      time.sleep(1)
    cls.serial_number = None



"""Module to handle launching a URL no matter the environment."""

import os
import sys
import webbrowser


def launch(url):
  """Attempt to launch the given URL."""
  if os.environ.get('DISPLAY') is not None:
    webbrowser.open(url)
    return


if __name__ == '__main__':
  launch(sys.argv[1])



"""
Simple util to dynamically load the conf help text for documentation.
"""
from .base import TestPlan # Import all stuff used by TestPlan so that conf variables are declared.
from openhtf.util import conf

class ConfigHelpText(object):
    pass

docstring = """
Attributes:
"""

for name, decl in conf._declarations.items():
    description = decl.description.replace('\n', ' ').strip() if decl.description else "(no description)"
    default_value = decl.default_value
    docstring += (f"""
    {name}: {description}\n
        Default Value= {default_value!r}
    """)

ConfigHelpText.__doc__ = docstring




# Copyright 2016 Google Inc. All Rights Reserved.

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#     http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Modified by Tack Verification, 2020

"""This module contains support for various built-in output mechanisms.

Here, a base OutputToFile class is implemented to provide simple output to
a file via the pickle serialization mechanism. It can be subclassed to implement
alternative serialization schemes, see json_factory.py and mfg_inspector.py for
examples.
"""

import os
import contextlib
try:
   import cPickle as pickle
except:
   import pickle
import shutil
import tempfile

from openhtf import util
from openhtf.util import data
import six


# TODO(wallacbe): Switch to util
class Atomic(object):
  """Class that does atomic write in a contextual manner."""

  def __init__(self, filename):
    self.filename = filename
    self.temp = tempfile.NamedTemporaryFile(delete=False)

  def write(self, write_data):
    if hasattr(write_data, 'decode'):
      return self.temp.write(write_data)
    return self.temp.write(write_data.encode())

  def close(self):
    self.temp.close()
    shutil.move(self.temp.name, self.filename)


class OutputToFile(object):
  """Output the given TestRecord to a file.

  Instances of this class are intended to be used as an output callback
  (see Test.add_output_callbacks) to output TestRecord results to a file.
  This base implementation outputs the TestRecord by serializing it via
  the pickle module.  Subclasses may change this by overriding the
  serialize_test_record() method.  Additionally, subclasses may implement
  more complex file naming mechanisms by overriding the open_file() method.

  Args:
    test_record: The TestRecord to write out to a file.
  """

  def __init__(self, filename_pattern):
    self.filename_pattern = filename_pattern

  @staticmethod
  def serialize_test_record(test_record):
    """Override method to alter how test records are serialized to file data."""
    return pickle.dumps(test_record, -1)

  @staticmethod
  def open_file(filename):
    """Override method to alter file open behavior or file types."""
    return Atomic(filename)

  @contextlib.contextmanager
  def open_output_file(self, test_record):
    """Open file based on pattern."""
    # Ignore keys for the log filename to not convert larger data structures.
    record_dict = data.convert_to_base_types(
        test_record, ignore_keys=('code_info', 'phases', 'log_records'))
    pattern = self.filename_pattern
    if isinstance(pattern, six.string_types) or callable(pattern):
      filepath = util.format_string(pattern, record_dict)
      output_folder, _ = os.path.split(filepath)
      
      if not os.path.exists(output_folder):
        os.makedirs(output_folder)
      
      output_file = self.open_file(filepath)
      
      try:
        yield output_file
      finally:
        output_file.close()
    elif hasattr(self.filename_pattern, 'write'):
      yield self.filename_pattern
    else:
      raise ValueError(
          'filename_pattern must be string, callable, or File-like object')

  def __call__(self, test_record):
    with self.open_output_file(test_record) as outfile:
      outfile.write(self.serialize_test_record(test_record))
